    
          SUBROUTINE ecosim(SpecIr,avcos)
!
!!======================================================================
!! April/May, 2007                                                     ! 
!!======================================================Marta Rodrigues=
!!                                                                     !
!! This subroutine is from ROMS                                        !                  	
!! The model was extend for zooplankton                                ! 
!!======================================================================
!************************************************** Hernan G. Arango ***
!  Copyright (c) 2002-2007 The ROMS/TOMS Group                         !
!    Licensed under a MIT/X style license                              !
!    See License_ROMS.txt                                              !
!*************************************************** W. Paul Bissett ***
!  Copyright (c) 1997 W. Paul Bissett, FERI                            !
!***********************************************************************
!                                                                      !
!  The EcoSim code has been developed for research purposes only. It   !
!  consists of unpublished, proprietary formulations protected under   !
!  U.S. copyright law. It is freely available on request from the      !
!  Florida Environmental Research Institute (FERI). Commercial usage   !
!  of these formulations is forbidden without express written          !
!  permission from FERI. All rights reserved.                          !
!                                                                      !
!************************************************** Hernan G. Arango ***
!                                                                      !
!  This routine computes the EcoSim sources and sinks and adds them    !
!  to the global biological fields.                                    !
!                                                                      !
!  Reference:                                                          !
!                                                                      !
!    Bissett, W.P., J.J. Walsh, D.A. Dieterle, K.L. Carder, 1999:      !
!      Carbon cycling in the upper waters of the Sargasso Sea: I.      !
!      Numerical  simulation of  differential carbon and nitrogen      !
!      fluxes,  Deep-Sea Res., 46, 205-269.                            !
!                                                                      !
!    Bissett, W.P., K.L. Carder, J.J. Walsh, D.A. Dieterle, 1999:      !
!      Carbon cycling in the upper waters of the Sargasso Sea: II.     !
!      Numerical  simulation  of  apparent  and  inherent optical      !
!      properties, Deep-Sea Res., 46, 271-317                          !
!                                                                      !
!  NOTES to EcoSim                                                     !
!  This version uses a descending index for depth that is different    !
!  than the original coding.                                           !
!                                                                      !
!***********************************************************************
!
!
      USE bio_param
      USE biology
      USE elfe_glbl, only : nea,nvrt,tsel,bdy_frc,flx_sf,flx_bt,idry_e,kbe,ze,tr_el,dt,ntracers2
      USE elfe_msgp, only : myrank,parallel_abort

      IMPLICIT NONE
      SAVE
!
!  Imported variable declarations.
!
!      integer, intent(in) :: nea, nvrt
      
!      real(r8), intent(in) :: Hz(nvrt,nea)
!      real(r8), intent(in) :: zr(nvrt,nea)
!      real(r8), intent(in) :: z_w(nvrt,nea)      
!      real(r8), intent(in) :: dt

      real(r8), intent(in) :: SpecIr(nea,NBands)
      real(r8), intent(in) :: avcos(nea,NBands)
      
!
!  Local variable declarations.
!
      integer :: Iter, Tindex, i, ibio, id, itrc, j, k
      integer :: ibac, iband, idom, ifec, iphy, ipig
!Marta Rodrigues
      integer :: izoo  
      
      real(r8), parameter :: MinVal = 0.0_r8

      real(r8) :: FV1, FV2, FV3, FV4, FV5, FV6, FV7, dtbio
      real(r8) :: DOC_lab, Ed_tot, Nup_max, aph442, aPHYN_wa
      real(r8) :: avgcos_min, par_b, par_bb, photo_DIC, photo_DOC
      real(r8) :: photo_decay, slope_AC, tChl, theta_m, total_photo
      real(r8) :: tot_ab, tot_b, tot_bb

      real(r8) :: Het_BAC
      real(r8) :: N_quota, RelDOC1, RelDON1, RelDOP1, RelFe

! Marta Rodrigues
      real(r8), dimension(nvrt,nea) :: Hz, zr

      real(r8), dimension(4) :: Bac_G

      real(r8), dimension(NBands) :: dATT_sum

      real(r8), dimension(nvrt,NBands) :: specir_d
      real(r8), dimension(nvrt,NBands) :: avgcos, dATT

      real(r8), dimension(nvrt,Nphy) :: C2CHL, C2CHL_w
      real(r8), dimension(nvrt,Nphy) :: Gt_fl, Gt_ll, Gt_nl
      real(r8), dimension(nvrt,Nphy) :: Gt_sl, Gt_pl 
      real(r8), dimension(nvrt,Nphy) :: alfa
      real(r8), dimension(nvrt,Nphy) :: pac_eff

      real(r8), dimension(nvrt,Nphy,Npig) :: Pigs_w

      integer, dimension(nea) :: Keuphotic

      real(r8), dimension(nvrt,nea) :: E0_nz
      real(r8), dimension(nvrt,nea) :: Ed_nz
      real(r8), dimension(nvrt,nea) :: DOC_frac
      real(r8), dimension(nvrt,nea) :: Hz_inv
      real(r8), dimension(nvrt,nea) :: NitrBAC
      real(r8), dimension(nvrt,nea) :: NH4toNO3
      real(r8), dimension(nvrt,nea) :: NtoNBAC
      real(r8), dimension(nvrt,nea) :: NtoPBAC
      real(r8), dimension(nvrt,nea) :: NtoFeBAC
      real(r8), dimension(nvrt,nea) :: totDOC_d
      real(r8), dimension(nvrt,nea) :: totDON_d
      real(r8), dimension(nvrt,nea) :: totDOP_d
      real(r8), dimension(nvrt,nea) :: totFe_d
      real(r8), dimension(nvrt,nea) :: totNH4_d
      real(r8), dimension(nvrt,nea) :: totNO3_d
      real(r8), dimension(nvrt,nea) :: totPO4_d
      real(r8), dimension(nvrt,nea) :: totSiO_d

      real(r8), dimension(nvrt,nea,Nbac) :: GtBAC
      real(r8), dimension(nvrt,nea,Nbac) :: NupDOC_ba
      real(r8), dimension(nvrt,nea,Nbac) :: NupDON_ba
      real(r8), dimension(nvrt,nea,Nbac) :: NupDOP_ba
      real(r8), dimension(nvrt,nea,Nbac) :: NupFe_ba
      real(r8), dimension(nvrt,nea,Nbac) :: NupNH4_ba
      real(r8), dimension(nvrt,nea,Nbac) :: NupPO4_ba

      real(r8), dimension(nvrt,nea,Nphy) :: C2fALG
      real(r8), dimension(nvrt,nea,Nphy) :: C2nALG
      real(r8), dimension(nvrt,nea,Nphy) :: C2pALG
      real(r8), dimension(nvrt,nea,Nphy) :: C2sALG
      real(r8), dimension(nvrt,nea,Nphy) :: GtALG
      real(r8), dimension(nvrt,nea,Nphy) :: GtALG_r
      real(r8), dimension(nvrt,nea,Nphy) :: NupDOP
      real(r8), dimension(nvrt,nea,Nphy) :: NupDON
      real(r8), dimension(nvrt,nea,Nphy) :: NupFe
      real(r8), dimension(nvrt,nea,Nphy) :: NupNH4
      real(r8), dimension(nvrt,nea,Nphy) :: NupNO3
      real(r8), dimension(nvrt,nea,Nphy) :: NupPO4
      real(r8), dimension(nvrt,nea,Nphy) :: NupSiO
      real(r8), dimension(nvrt,nea,Nphy) :: graz_act
      real(r8), dimension(nvrt,nea,Nphy) :: mu_bar_f
      real(r8), dimension(nvrt,nea,Nphy) :: mu_bar_n
      real(r8), dimension(nvrt,nea,Nphy) :: mu_bar_p
      real(r8), dimension(nvrt,nea,Nphy) :: mu_bar_s
      real(r8), dimension(nvrt,nea,Nphy) :: refuge

      real(r8), dimension(nvrt,nea,Nfec) :: Regen_C
      real(r8), dimension(nvrt,nea,Nfec) :: Regen_F
      real(r8), dimension(nvrt,nea,Nfec) :: Regen_N
      real(r8), dimension(nvrt,nea,Nfec) :: Regen_P
      real(r8), dimension(nvrt,nea,Nfec) :: Regen_S

      real(r8), dimension(nvrt,nea,NBands) :: specir_scal
      real(r8), dimension(nvrt,nea,Nphy,NBands) :: aPHYN_al
      real(r8), dimension(nvrt,nea,Nphy,NBands) :: aPHYN_at

      real(r8), dimension(nvrt,nea,ntracers2) :: Bio
      real(r8), dimension(nvrt,nea,ntracers2) :: Bio_old
      real(r8), dimension(nvrt,nea,ntracers2) :: Bio_new

! Marta Rodrigues
      real(r8), dimension(nvrt,nea,Nzoo) :: GtZOO
      real(r8), dimension(nvrt,nea,Nzoo) :: Gt_z
      real(r8), dimension(nvrt,nea,Nzoo) :: foodC
      real(r8), dimension(nvrt,nea,Nzoo) :: foodN
      real(r8), dimension(nvrt,nea,Nzoo) :: foodP
      real(r8), dimension(nvrt,nea,Nzoo,Nphy) :: Gt_zoo_phy
            
!
!=======================================================================
!  Add EcoSim Source/Sink terms.
!=======================================================================
!
      !! MFR - not used   J_LOOP : DO j=Jstr,Jend

! Marta Rodrigues
! ----------------------------------------------------------------------
! Compute thickness and actual depths at RHO-points (in the middle of
! the volume of control)
! ----------------------------------------------------------------------

        DO i=1, nea
          if(idry_e(i)==1) cycle

          DO k=kbe(i)+1, nvrt
              Hz(k,i)=ze(k,i)-ze(k-1,i)
          END DO
        END DO ! element

        DO i=1, nea
          if(idry_e(i)==1) cycle
          DO k=kbe(i)+1, nvrt
            zr(k,i)=((ze(k-1,i)-ze(k,i))/2)+ze(k,i)
          END DO
        END DO ! element
!
!-----------------------------------------------------------------------
!  Compute inverse thickness to avoid repeated divisions.
!-----------------------------------------------------------------------
!
        DO i=1,nea
          if(idry_e(i)==1) cycle
          DO k=kbe(i)+1,nvrt 
            Hz_inv(k,i)=1.0_r8/Hz(k,i)
          END DO
        END DO

!
!-----------------------------------------------------------------------
!  Extract biological variables from tracer arrays, place them into
!  scratch arrays, and restrict their values to be positive definite.
!-----------------------------------------------------------------------
!

! MFR - Feb/2008

        Bio=0.0_r8
        Bio_old=0.0_r8

        DO ibio=1,NBT
          itrc=idbio(ibio)
          DO i=1,nea
            if(idry_e(i)==1) cycle
            DO k=kbe(i)+1,nvrt               
!MFR              Bio(k,i,itrc)=MAX(MinVal,tr_el(itrc,k,i)*Hz_inv(k,i))
              Bio(k,i,itrc)=MAX(MinVal,tr_el(itrc,k,i))
              Bio_old(k,i,itrc)=Bio(k,i,itrc)

!! HGA - The new tendency terms were not initialized.  This gives
!!       unexpected behavior on different computers since a variable
!!       was used before it was assigned.  This may explain earlier
!!       problems with the algorithm.  Perhaps, this time-stepping can
!!       be modified latter to avoid unnecessary storage between
!!       Bio, Bio_old, and Bio_new.
!!
              Bio_new(k,i,itrc)=0.0_r8
            END DO
          END DO
        END DO
!
!  Extract temperature and salinity - MFR.
!
        DO i=1,nea
          if(idry_e(i)==1) cycle
          DO k=kbe(i)+1,nvrt
            Bio(k,i,itemp)=tsel(1,k,i)  !MFR *Hz_inv(k,i)
            Bio(k,i,isalt)=tsel(2,k,i)  !MFR *Hz_inv(k,i)
          END DO
        END DO

!
!-----------------------------------------------------------------------
!  Compute temperature and salinity dependent variables.
!-----------------------------------------------------------------------
!
!  Refuge depth calculation.
!
        DO iphy=1,Nphy
          DO i=1,nea
            if(idry_e(i)==1) cycle
            DO k=kbe(i)+1,nvrt
              refuge(k,i,iphy)=MinRefuge(iphy)
            END DO
          END DO
        END DO
!
!  Initialize fecal regeneration arrays (N, P, and Fe from Moore et al.,
!  DSRII 2001; silica is given by values from Bidle and Azam, Nature,
!  1999).
!
        IF (Regen_flag) THEN
          DO ifec=1,Nfec
            DO i=1,nea
              if(idry_e(i)==1) cycle
              DO k=kbe(i)+1,nvrt
                FV1=EXP(RegTfac(ifec)*(Bio(k,i,itemp)-               &
     &                  RegTbase(ifec)))
                Regen_C(k,i,ifec)=RegCR(ifec)*FV1
                Regen_N(k,i,ifec)=RegNR(ifec)*FV1
                Regen_P(k,i,ifec)=RegPR(ifec)*FV1
                Regen_F(k,i,ifec)=RegFR(ifec)*FV1
                Regen_S(k,i,ifec)=RegSR(ifec)*FV1
              END DO
            END DO
          END DO
        END IF
!
!  Calculate temperature dependent growth rate.
!
        DO iphy=1,Nphy
          DO i=1,nea
            if(idry_e(i)==1) cycle
            DO k=kbe(i)+1,nvrt
              GtALG(k,i,iphy)=GtALG_max(iphy)*                       &
     &                        EXP(PhyTfac(iphy)*                     &
     &                            (Bio(k,i,itemp)-PhyTbase(iphy)))

!
!  Calculate mu_bar for droop equation.
!
              FV1=maxC2nALG(iphy)*(1.0_r8+GtALG(k,i,iphy))
              mu_bar_n(k,i,iphy)=GtALG(k,i,iphy)*                       &
     &                           FV1/(FV1-minC2nALG(iphy))
              IF (HsSiO(iphy).lt.LARGE) THEN
                FV1=maxC2SiALG(iphy)*(1.0_r8+GtALG(k,i,iphy))
                mu_bar_s(k,i,iphy)=GtALG(k,i,iphy)*                     &
     &                             FV1/(FV1-minC2SiALG(iphy))
              ELSE
                mu_bar_s(k,i,iphy)=LARGE
              END IF
              IF (HsPO4(iphy).lt.LARGE) THEN
                FV1=maxC2pALG(iphy)*(1.0_r8+GtALG(k,i,iphy))
                mu_bar_p(k,i,iphy)=GtALG(k,i,iphy)*                     &
     &                             FV1/(FV1-minC2pALG(iphy))
              ELSE
                mu_bar_p(k,i,iphy)=LARGE
              END IF
              IF (HsFe(iphy).lt.LARGE) THEN
                FV1=maxC2FeALG(iphy)*(1.0_r8+GtALG(k,i,iphy))
                mu_bar_f(k,i,iphy)=GtALG(k,i,iphy)*                     &
     &                             FV1/(FV1-minC2FeALG(iphy))
              ELSE
                mu_bar_f(k,i,iphy)=LARGE
              END IF
            END DO
          END DO
        END DO

!
!  Bacterial growth rate from Fasham et al., 1990.
!
        DO ibac=1,Nbac
          DO i=1,nea
            if(idry_e(i)==1) cycle
            DO k=kbe(i)+1,nvrt            
              GtBAC(k,i,ibac)=GtBAC_max(ibac)*                       &
     &                        EXP(BacTfac(ibac)*                     &
     &                            (Bio(k,i,itemp)-BacTbase(ibac)))
            END DO
          END DO
        END DO
!
!  Grazing rate calculation.
!  NOTE: ES1 included separation calculations for grazing beneath the
!        zone of refuge (250 m). This has been removed and may
!        result in differences in deeper waters.
!! Revisions, WPB 10/20/02.  New grazing formulation that is better
!! representation of basal loss rates and biomass accumulations.
!

! Marta Rodrigues
! This term has been changed to account for the new variable zooplankton
! Now HsGRZ represents only the natural mortality (lises) 
! of the the phytoplankton
        
!	DO iphy=1,Nphy
!          DO k=kbe(i)+1,N(ng)
!            DO i=Istr,Iend
!              FV1=MAX(1.0_r8,(Bio(k,i,iPhyC(iphy))/refuge(k,i,iphy)))
!              graz_act(k,i,iphy)=HsGRZ(iphy,ng)*LOG(FV1)
!            END DO
!          END DO
!        END DO
! 

! Marta Rodrigues
! Zooplankton growth rate
        
        DO izoo=1, Nzoo
          DO i=1,nea
            if(idry_e(i)==1) cycle
            DO k=kbe(i)+1, nvrt
              IF(zoo_sp(izoo)==1)THEN !Acartia tonsa, naupilus
	        GtZOO(k,i,izoo)=0.0914_r8*EXP(0.0701_r8*Bio(k,i,itemp))&
                                *sec2day        
              ELSE IF(zoo_sp(izoo)==2)THEN !Acartia tonsa, copepodites
	        GtZOO(k,i,izoo)=0.0517_r8*EXP(0.130_r8*Bio(k,i,itemp))&
                                *sec2day
	      ELSE IF(zoo_sp(izoo)==3)THEN !Acartia clausi, naupilus
	        GtZOO(k,i,izoo)=0.0591_r8*EXP(0.0775_r8*Bio(k,i,itemp))&
                                *sec2day 
	      ELSE IF(zoo_sp(izoo)==4)THEN !Acartia clausi, copepodites
	        GtZOO(k,i,izoo)=0.0435_r8*EXP(0.114_r8*Bio(k,i,itemp))&
                                *sec2day
	      ELSE
	         call parallel_abort('Bad flag for zooplankton maximum growth rate calculation!')
!'
	      END IF !zoo_sp
	    END DO
	  END DO
	END DO      
	    
    
!-----------------------------------------------------------------------
!  Iterate biology source and sink terms.
!-----------------------------------------------------------------------
!
        dtbio=dt/REAL(BioIter,r8)

        ITER_LOOP : DO Iter=1,BioIter
 
          DO i=1,nea
            if(idry_e(i)==1) cycle
            DO k=kbe(i)+1,nvrt                       
              totNH4_d(k,i)=0.0_r8
              totNO3_d(k,i)=0.0_r8
              totPO4_d(k,i)=0.0_r8
              totSiO_d(k,i)=0.0_r8
              totFe_d (k,i)=0.0_r8
              totDOC_d(k,i)=0.0_r8
              totDON_d(k,i)=0.0_r8
              totDOP_d(k,i)=0.0_r8
            END DO
          END DO
          DO iphy=1,Nphy
            DO i=1,nea
              if(idry_e(i)==1) cycle
              DO k=kbe(i)+1,nvrt              
                NupNH4(k,i,iphy)=0.0_r8
                NupNO3(k,i,iphy)=0.0_r8
                NupPO4(k,i,iphy)=0.0_r8
                NupSiO(k,i,iphy)=0.0_r8
                NupFe (k,i,iphy)=0.0_r8
                NupDON(k,i,iphy)=0.0_r8
                NupDOP(k,i,iphy)=0.0_r8
              END DO
            END DO
          END DO
!
!  Compute Ratio Arrays.
!  (Calculating only those that are accessed more than once.)
!
          DO iphy=1,Nphy
            DO i=1,nea
              if(idry_e(i)==1) cycle
              DO k=kbe(i)+1,nvrt
                C2nALG(k,i,iphy)=0.0_r8
                IF (Bio(k,i,iPhyN(iphy)).gt.0.0_r8) THEN
                  C2nALG(k,i,iphy)=Bio(k,i,iPhyC(iphy))/                &
     &                             Bio(k,i,iPhyN(iphy))
                END IF
                C2pALG(k,i,iphy)=0.0_r8
                IF (Bio(k,i,iPhyP(iphy)).gt.0.0_r8) THEN
                  C2pALG(k,i,iphy)=Bio(k,i,iPhyC(iphy))/                &
     &                             Bio(k,i,iPhyP(iphy))
                END IF 
                C2sALG(k,i,iphy)=0.0_r8
                IF (iPhyS(iphy).gt.0) THEN
                  IF (Bio(k,i,iPhyS(iphy)).gt.0.0_r8) THEN
                    C2sALG(k,i,iphy)=Bio(k,i,iPhyC(iphy))/              &
     &                               Bio(k,i,iPhyS(iphy))
                  END IF
                END IF 

! MFR - Only if Fe is being calculated...

                IF(IRON==1)THEN
                  C2fALG(k,i,iphy)=0.0_r8
                  IF (Bio(k,i,iPhyF(iphy)).gt.0.0_r8) THEN
                    C2fALG(k,i,iphy)=Bio(k,i,iPhyC(iphy))/                &
     &                               Bio(k,i,iPhyF(iphy))
                  END IF
                END IF  !IRON 
             
              END DO
            END DO
          END DO
!


!  Marta Rodrigues
!-----------------------------------------------------------------------
!  Compute zooplankton growth rate
!-----------------------------------------------------------------------	
		    
!  Compute food available for zooplankton
         
          DO izoo=1,Nzoo
            DO i=1,nea
              if(idry_e(i)==1) cycle
              DO k=kbe(i)+1, nvrt            
                foodC(k,i,izoo)=0.0_r8
	        foodN(k,i,izoo)=0.0_r8
	        foodP(k,i,izoo)=0.0_r8
	        Gt_z(k,i,izoo)=0.0_r8
	      END DO
	    END DO
	  END DO
   
          DO izoo=1,Nzoo
            DO i=1,nea
              if(idry_e(i)==1) cycle
              DO k=kbe(i)+1, nvrt                
                DO iphy=1,Nphy
                  foodC(k,i,izoo)=foodC(k,i,izoo)+DeltaZoo(izoo,iphy)*EfcCap(izoo,iphy) &
		                                  *Bio(k,i,iphyC(iphy))
                  foodN(k,i,izoo)=foodN(k,i,izoo)+DeltaZoo(izoo,iphy)*EfcCap(izoo,iphy) &
		                                  *Bio(k,i,iphyN(iphy))
                  foodP(k,i,izoo)=foodP(k,i,izoo)+DeltaZoo(izoo,iphy)*EfcCap(izoo,iphy) &
		                                  *Bio(k,i,iphyP(iphy))
                END DO
              END DO  
            END DO
          END DO  
       
!  Growth rate
          DO izoo=1,Nzoo
            DO i=1,nea
              if(idry_e(i)==1) cycle
              DO k=kbe(i)+1,nvrt
                DO iphy=1,Nphy
		  FV1=GtZOO(k,i,izoo)*DeltaZoo(izoo,iphy)*EfcCap(izoo,iphy) &
		      *Bio(k,i,iphyC(iphy))/foodC(k,i,izoo)
		  FV2=foodC(k,i,izoo)/(foodC(k,i,izoo)+HsZoo(izoo))
                  Gt_zoo_phy(k,i,izoo,iphy)=FV1*FV2
                  Gt_z(k,i,izoo)=Gt_z(k,i,izoo)+FV1*FV2
                END DO
              END DO
            END DO   
          END DO

!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!  Daylight Computations.
!-----------------------------------------------------------------------
!
! Initialize.
!
          DO i=1,nea
            if(idry_e(i)==1) cycle
            Ed_nz(i,nvrt)=0.0_r8
            E0_nz(i,nvrt)=0.0_r8
            Keuphotic(i)=nvrt+1
            IF (SpecIr(i,21).gt.VSMALL) THEN
              DO k=kbe(i)+1,nvrt-1
                Ed_nz(k,i)=0.0_r8
                E0_nz(k,i)=0.0_r8
              END DO
              DO iband=1,NBands
                dATT_sum(iband)=0.0_r8
                DO k=kbe(i)+1,nvrt
                  dATT(k,iband)=0.0_r8
                END DO
                DO iphy=1,Nphy
                  DO k=kbe(i)+1,nvrt
                    aPHYN_at(k,i,iphy,iband)=0.0_r8
                    aPHYN_al(k,i,iphy,iband)=0.0_r8
                  END DO
                END DO
              END DO
!
!  Calculate average cosine zenith angle at surface.
!  (See equation 14 Morel, 1991 Prog. Ocean.)
!
              Ed_tot=0.0_r8
              DO iband=1,NBands
                Ed_tot=Ed_tot+SpecIr(i,iband)*DLAM
                avgcos(nvrt,iband)=avcos(i,iband)
              END DO
!
!  Total aph(442). adp(442) is set to 50% of aph(442).
!  NOTE: choosing sbands=9 which is band 442 using v8r16
!        sbands formulation. If spectral resolution changes, this
!        value must change!
!
              DO k=nvrt,kbe(i+1),-1
                IF (Ed_tot.ge.1.0_r8) THEN
                  aph442=0.0_r8
                  tChl=0.0_r8
                  DO iphy=1,Nphy
                    IF (Bio(k,i,iPhyC(iphy)).gt.0.0_r8) THEN
                      tChl=tChl+Bio(k,i,iPigs(iphy,ichl))
                      pac_eff(k,iphy)=1.0_r8
                      IF (b_PacEff(iphy).gt.SMALL) THEN
                        FV2=Bio(k,i,iPigs(iphy,ichl))/                  &
     &                      (Bio(k,i,iPhyC(iphy))*12.0_r8)
                        pac_eff(k,iphy)=MAX(0.5_r8,                     &
     &                                      (MIN(1.0_r8,                &
     &                                           b_PacEff(iphy)+     &
     &                                           mxPacEff(iphy)*     &
     &                                           (FV2-                  &
     &                                            b_C2Cl(iphy)))))
                      END IF
                      iband=9
                      DO ipig=1,Npig
                        IF (iPigs(iphy,ipig).gt.0) THEN
                          aph442=aph442+                                &
     &                           Bio(k,i,iPigs(iphy,ipig))*             &
     &                           apigs(ipig,iband)*pac_eff(k,iphy)
                        END IF
                      END DO
                    END IF
                  END DO
!
!  Calculate absorption.
!  Calculating phytoplankton absorption for attentuation calculation.
!  NOTE: 12 factor to convert to ugrams (mg m-3)
!
                  aph442=0.5_r8*aph442
                  DO iband=1,NBands
                    tot_ab=0.0_r8
                    DO iphy=1,Nphy
                      DO ipig=1,Npig 
                        IF (iPigs(iphy,ipig).gt.0) THEN
                          aPHYN_at(k,i,iphy,iband)=                     &
     &                                      aPHYN_at(k,i,iphy,iband)+   &
     &                                      Bio(k,i,iPigs(iphy,ipig))*  &
     &                                      apigs(ipig,iband)*          &
     &                                      pac_eff(k,iphy)
                        END IF
                      END DO
                      tot_ab=tot_ab+aPHYN_at(k,i,iphy,iband)
!
!  Removing absorption due to PPC for "alfa" calculation.
!
                      ipig=5
                      IF (iPigs(iphy,ipig).gt.0) THEN
                        aPHYN_al(k,i,iphy,iband)=                       &
     &                                    aPHYN_at(k,i,iphy,iband)-     &
     &                                    Bio(k,i,iPigs(iphy,ipig))*    &
     &                                    apigs(ipig,iband)*            &
     &                                    pac_eff(k,iphy)
                      END IF
                    END DO
!
!  Adding detrital absorption.
!
                    tot_ab=tot_ab+                                      &
     &                     aph442*EXP(0.011_r8*(442.0_r8-               &
     &                                (397.0_r8+REAL(iband,r8)*DLAM)))
!
! MFR - Only if CDOC is being calculated

                    IF(CDOC==1)THEN 
!
!  Calculate CDOC absorption.
!  NOTE: 12 factor is to convert ugrams per liter, and 0.001 converts
!        to mg/liter.  Specific absorption
!        coefficients were calculated as m-1 / (mg DOC/liters sw).
!        net factor = (12*0.001) = 0.012
!
                     tot_ab=tot_ab+                                      &
     &                      0.012_r8*(Bio(k,i,iCDMC(ilab))*              &
     &                                aDOC(ilab,iband)+                  &
     &                                Bio(k,i,iCDMC(irct))*              &
     &                                aDOC(irct,iband))+                 &
     &                      awater(iband)

                     END IF  !CDOC=1
!
!  Calculate scattering and backscattering (see equation 19 Morel, 1991,
!  Prog. Ocean). Morel, 1988 puts spectral dependency in backscattering.
!  Since Morel (1991) does not have a backscattering equation, use 1988
!  paper. Morel 2001 has slight adjustment 0.01, rather than 0.02. 
!  This was altered, but never tested in ROMS 1.8 on 03/08/03.
!
                    par_b =0.3_r8*(tChl**0.62_r8)
                    par_bb=0.0_r8
                    IF (tChl.gt.0.0_r8) THEN
                      par_bb=par_b*(0.002_r8+0.01_r8*                   &
     &                              (0.5_r8-0.25_r8*LOG10(tChl))*       &
     &                              wavedp(iband))
                    END IF
                    par_bb=MAX(par_bb,0.0_r8)
!
!  However, for omega0 calculation, par_b must be spectral, so use
!  dependency from Sathy and Platt 1988
!
                    tot_b=bwater(iband)+par_b*wavedp(iband)
!
!  Morel, 1988 instead of 1991. See methods
!
                    tot_bb=0.5_r8*bwater(iband)+par_bb
!
!  Sathy and Platt JGR 1988.  This is set with the average cosine of
!  the box above, and used to calculate a new avgcos for this level.
!  This new average cosine is then used to recalculate the attenuation
!  coefficient
!
                    dATT(k,iband)=(tot_ab+tot_bb)/avgcos(k,iband)
!
!  See Mobley, 1995 for graphical depiction of this equation.
!
                    avgcos_min=avgcos(k,iband)+                         &
     &                         (0.5_r8-avgcos(k,iband))*                &
     &                         (tot_b/(tot_ab+tot_b))
!
!  Calculate average cosine. Linear fit to average cosine versus optical
!  depth relationship. The FV1 calculation keeps the denominator of the
!  slope calculation from going negative and above 1.
!
                    FV1=MAX(1.0_r8,                                     &
     &                      7.0_r8-dATT(k,iband)*ABS(zr(k,i)))
                    slope_AC =MIN(0.0_r8,                               &
     &                            (avgcos_min-avgcos(k,iband))/FV1)
                    avgcos(k,iband)=avgcos(k,iband)+                    &
     &                             slope_AC*dATT(k,iband)*Hz(k,i)
                    dATT(k,iband)=(tot_ab+tot_bb)/avgcos(k,iband)
!
!  Set avgcos for next level.
!
                    IF (k.ne.1) THEN
                      avgcos(k-1,iband)=avgcos(k,iband)
                    END IF
!
!  Calculate spectral irradiance with depth.
!
                    FV1=dATT(k,iband)*Hz(k,i)
                    FV2=dATT_sum(iband)+0.5_r8*FV1
                    dATT_sum(iband)=dATT_sum(iband)+FV1
                    specir_d(k,iband)=SpecIr(i,iband)*                &
     &                                EXP(-FV2)*DLAM
!
!  Calculate spectral scalar irradiance.  Morel, 1991 Prog. Ocean.
!
                    specir_scal(k,i,iband)=specir_d(k,iband)*           &
     &                                     (dATT(k,iband)/tot_ab)
                    E0_nz(k,i)=E0_nz(k,i)+specir_scal(k,i,iband)
!
!  Calculate Ed_nz.
!
                    Ed_nz(k,i)=Ed_nz(k,i)+specir_d(k,iband)
                  END DO
                  Ed_tot=E0_nz(k,i)
!
!  Set bottom of the euphotic zone.
!
                  Keuphotic(i)=k
                END IF
              END DO
            END IF
          END DO
!
!-----------------------------------------------------------------------
!  Bacterial nutrient uptake.
!-----------------------------------------------------------------------
!
          DO ibac=1,Nbac
            DO i=1,nea
              if(idry_e(i)==1) cycle
              DO k=kbe(i)+1,nvrt              
!
!  DOM uptake.
!
                IF ((Bio(k,i,iDOMC(ilab)).gt.0.0_r8).and.               &
     &              (Bio(k,i,iDOMN(ilab)).gt.0.0_r8).and.               &
     &              (Bio(k,i,iDOMP(ilab)).gt.0.0_r8)) THEN
                  NupDOC_ba(k,i,ibac)=GtBAC(k,i,ibac)*                  &
     &                                Bio(k,i,iBacC(ibac))*             &
     &                                I_Bac_Ceff*                       &
     &                                (Bio(k,i,iDOMC(ilab))/            &
     &                                (HsDOC_ba(ibac)+                  &
     &                                 Bio(k,i,iDOMC(ilab))))
                  NupDON_ba(k,i,ibac)=NupDOC_ba(k,i,ibac)*              &
     &                                Bio(k,i,iDOMN(ilab))/             &
     &                                Bio(k,i,iDOMC(ilab))
                  NupDOP_ba(k,i,ibac)=NupDOC_ba(k,i,ibac)*              &
     &                                Bio(k,i,iDOMP(ilab))/             &
     &                                Bio(k,i,iDOMC(ilab))
                ELSE
                  NupDOC_ba(k,i,ibac)=0.0_r8
                  NupDON_ba(k,i,ibac)=0.0_r8
                  NupDOP_ba(k,i,ibac)=0.0_r8
                END IF
                totDOC_d(k,i)=totDOC_d(k,i)+NupDOC_ba(k,i,ibac)
                totDON_d(k,i)=totDON_d(k,i)+NupDON_ba(k,i,ibac)
                totDOP_d(k,i)=totDOP_d(k,i)+NupDOP_ba(k,i,ibac)
!
!  NH4 uptake.
!
                NupNH4_ba(k,i,ibac)=GtBAC(k,i,ibac)*                    &
     &                              Bio(k,i,iBacN(ibac))*               &
     &                              Bio(k,i,iNH4_)/                     &
     &                              (HsNH4_ba(ibac)+Bio(k,i,iNH4_))
                totNH4_d(k,i)=totNH4_d(k,i)+NupNH4_ba(k,i,ibac)
!
!  PO4 uptake.
!
                NupPO4_ba(k,i,ibac)=GtBAC(k,i,ibac)*                    &
     &                              Bio(k,i,iBacP(ibac))*               &
     &                              Bio(k,i,iPO4_)/                     &
     &                              (HsPO4_ba(ibac)+Bio(k,i,iPO4_))
                totPO4_d(k,i)=totPO4_d(k,i)+NupPO4_ba(k,i,ibac)
!

!  MFR - Only if Fe is being calculated
               
                IF(IRON==1)THEN

!  Fe uptake.
!
                  NupFe_ba(k,i,ibac)=GtBAC(k,i,ibac)*                     &
     &                               Bio(k,i,iBacF(ibac))*                &
     &                               Bio(k,i,iFeO_)/                      &
     &                               (HsFe_ba(ibac)+Bio(k,i,iFeO_))
                  totFe_d(k,i)=totFe_d(k,i)+NupFe_ba(k,i,ibac)

                END IF  !IRON
 
              END DO
            END DO
          END DO
!
!-----------------------------------------------------------------------
!  Phytoplankton dark nutrient uptake.
!-----------------------------------------------------------------------
!
          DO iphy=1,Nphy
            DO i=1,nea
              if(idry_e(i)==1) cycle
              DO k=kbe(i)+1,nvrt              
                IF (C2nALG(k,i,iphy).gt.C2nALGminABS(iphy)) THEN
!
!  NOTE: these are being saved to test for total nutrient uptake.
!        If nutrient uptake is greater than maximum nutrient, then
!        each of the uptakes are reduced by their fractional contri-
!        bution to the total.
!
                  Nup_max=GtALG(k,i,iphy)
                  NupNO3(k,i,iphy)=(Bio(k,i,iNO3_)/                     &
     &                             (HsNO3(iphy)+Bio(k,i,iNO3_))*        &
     &                             EXP(-BET_(iphy)*Bio(k,i,iNH4_)))
                  NupNH4(k,i,iphy)=Bio(k,i,iNH4_)/                      &
     &                             (HsNH4(iphy)+Bio(k,i,iNH4_))
!
!  Test that Wroblewski equation does not exceed 1.0.
!
                  FV1=NupNO3(k,i,iphy)+NupNH4(k,i,iphy)
                  IF (FV1.gt.1.0_r8) THEN
                    FV1=1.0_r8/FV1
                    NupNO3(k,i,iphy)=NupNO3(k,i,iphy)*FV1
                    NupNH4(k,i,iphy)=NupNH4(k,i,iphy)*FV1
                  END IF
!
!  Change from percentage of maximum to mass per second.
!
                  FV1=Nup_max*Bio(k,i,iPhyN(iphy))
                  NupNO3(k,i,iphy)=NupNO3(k,i,iphy)*FV1
                  NupNH4(k,i,iphy)=NupNH4(k,i,iphy)*FV1
!
!  Test for DON uptake.
!
                  IF (C2nALG(k,i,iphy).gt.C2nNupDON(iphy)) THEN
                    NupDON(k,i,iphy)=FV1*                               &
     &                               Bio(k,i,iDOMN(ilab))/              &
     &                               (HsDON(iphy)+                      &
     &                                Bio(k,i,iDOMN(ilab)))
                  END IF
!
!  Accumulate total demand for nutrients.
!
                  totNO3_d(k,i)=totNO3_d(k,i)+NupNO3(k,i,iphy)
                  totNH4_d(k,i)=totNH4_d(k,i)+NupNH4(k,i,iphy)
                  totDON_d(k,i)=totDON_d(k,i)+NupDON(k,i,iphy)
                END IF
!
!  Dark silica uptake, min C2Si test.
!  The LARGE test can be removed after testing phase.
!
                IF (HsSiO(iphy).lt.LARGE) THEN
                  IF (C2sALG(k,i,iphy).gt.C2SiALGminABS(iphy)) THEN
                    Nup_max=GtALG(k,i,iphy)
                    NupSiO(k,i,iphy)=Bio(k,i,iSiO_)/                    &
     &                               (HsSiO(iphy)+Bio(k,i,iSiO_))
!
!  Change from percentage of maximum to mass per second.
!
                    IF (iPhyS(iphy).gt.0) THEN
                      FV1=Nup_max*Bio(k,i,iPhyS(iphy))
                      NupSiO(k,i,iphy)=NupSiO(k,i,iphy)*FV1
                    ELSE
                      NupSiO(k,i,iphy)=0.0_r8
                    END IF
!
!  Accumulate total demand for nutrients.
!
                    totSiO_d(k,i)=totSiO_d(k,i)+NupSiO(k,i,iphy)
                  END IF
                END IF
!
!  Dark phophorus uptake, min C2P test.
!  The LARGE test can be removed after testing phase.
!
                IF (HsPO4(iphy).lt.LARGE) THEN
                  IF (C2pALG(k,i,iphy).gt.C2pALGminABS(iphy)) THEN
                    Nup_max=GtALG(k,i,iphy)
                    NupPO4(k,i,iphy)=Bio(k,i,iPO4_)/                    &
     &                               (HsPO4(iphy)+Bio(k,i,iPO4_))
!
!  Change from percentage of maximum to mass per second.
!
                    FV1=Nup_max*Bio(k,i,iPhyP(iphy))
                    NupPO4(k,i,iphy)=NupPO4(k,i,iphy)*FV1
!
!  Test for alk. phosphatase
!
                    IF (C2pALG(k,i,iphy).gt.C2pALKPHOS(iphy)) THEN
                      NupDOP(k,i,iphy)=FV1*                             &
                                       Bio(k,i,iDOMP(ilab))/            &
     &                                 (HsDOP(iphy)+                 &
     &                                  Bio(k,i,iDOMP(ilab)))
                    END IF
!
!  Accumulate total demand for nutrients.
!
                    totPO4_d(k,i)=totPO4_d(k,i)+NupPO4(k,i,iphy)
                    totDOP_d(k,i)=totDOP_d(k,i)+NupDOP(k,i,iphy)
                  END IF
                END IF
!
!  MFR - Only if iron is being calculated...
                 
                IF(IRON==1)THEN

!  Dark iron uptake, min C2Fe test.
!  The LARGE test can be removed after testing phase.
!
                 IF (HsFe(iphy).lt.LARGE) THEN
                   IF (C2fALG(k,i,iphy).gt.C2FeALGminABS(iphy)) THEN
                     Nup_max=GtALG(k,i,iphy)
                     NupFe(k,i,iphy)=Bio(k,i,iFeO_)/                     &
     &                               (HsFe(iphy)+Bio(k,i,iFeO_))
!
!  Change from percentage of maximum to mass per second.
!
                     FV1=Nup_max*Bio(k,i,iPhyF(iphy))
                     NupFe(k,i,iphy)=NupFe(k,i,iphy)*FV1
!
!  Accumulate total demand for nutrients.
!
                     totFe_d(k,i)=totFe_d(k,i)+NupFe(k,i,iphy)
                   END IF
                 END IF
               
               END IF  !IRON
              END DO
            END DO
          END DO
!
!  Calculate bacterial nitrification as a Michaelis-Menton function
!  of ambient NH4 concentration, beneath the euphotic zone (light
!  inhibits nitrification).
!
          DO i=1,nea
            if(idry_e(i)==1) cycle
            DO k=kbe(i)+1,nvrt
              NitrBAC(k,i)=0.0_r8
              NH4toNO3(k,i)=0.0_r8
              NtoNBAC(k,i)=0.0_r8
              NtoPBAC(k,i)=0.0_r8
              NtoFeBAC(k,i)=0.0_r8
              IF (k.lt.Keuphotic(i)) THEN
                NH4toNO3(k,i)=RtNIT*                                &
     &                        Bio(k,i,iNH4_)/(HsNIT+Bio(k,i,iNH4_))
!
!  Nitrification fixes DIC into POC.
!  Conversion factor of 7.0 from Kaplan 1983 "Nitrogen in the Sea"
!  factor equals (1.0 / (7.0 * C2nBAC)). Adds NH4 uptake as biomass.
!
                NitrBAC(k,i)=NH4toNO3(k,i)/7.0_r8
                NtoNBAC(k,i)=NitrBAC(k,i)*N2cBAC
                NtoPBAC(k,i)=NitrBAC(k,i)*P2cBAC
                NtoFeBAC(k,i)=NitrBAC(k,i)*Fe2cBAC
                totNH4_d(k,i)=totNH4_d(k,i)+NH4toNO3(k,i)+NtoNBAC(k,i)
                totPO4_d(k,i)=totPO4_d(k,i)+NtoPBAC(k,i)
                totFe_d (k,i)=totFe_d (k,i)+NtoFeBAC(k,i)
              END IF
            END DO
          END DO
!
!-----------------------------------------------------------------------
!  Test that total nutrient demand does not exceed supply.  If it does
!  total demand is normalized to the total supply. Each species demand
!  is reduced to its weighted average percentage of the supply.
!-----------------------------------------------------------------------
!
          DO i=1,nea
            if(idry_e(i)==1) cycle
            DO k=kbe(i)+1,nvrt            
              FV2=totNO3_d(k,i)*dtbio
              IF (FV2.gt.Bio(k,i,iNO3_)) THEN
                FV1=(Bio(k,i,iNO3_)-VSMALL)/FV2
                DO iphy=1,Nphy
                  NupNO3(k,i,iphy)=NupNO3(k,i,iphy)*FV1
                END DO
              END IF
!
              FV2=totNH4_d(k,i)*dtbio
              IF (FV2.gt.Bio(k,i,iNH4_)) THEN
                FV1=(Bio(k,i,iNH4_)-VSMALL)/FV2
                DO iphy=1,Nphy
                  NupNH4(k,i,iphy)=NupNH4(k,i,iphy)*FV1
                END DO
                DO ibac=1,Nbac
                  NupNH4_ba(k,i,ibac)=NupNH4_ba(k,i,ibac)*FV1
                END DO
                NH4toNO3(k,i)=NH4toNO3(k,i)*FV1
                NtoNBAC(k,i)=NtoNBAC(k,i)*FV1
              END IF
!
              FV2=totSiO_d(k,i)*dtbio
              IF (FV2.gt.Bio(k,i,iSiO_)) THEN
                FV1=(Bio(k,i,iSiO_)-VSMALL)/FV2
                DO iphy=1,Nphy
                  NupSiO(k,i,iphy)=NupSiO(k,i,iphy)*FV1
                END DO
              END IF
!
              FV2=totPO4_d(k,i)*dtbio
              IF (FV2.gt.Bio(k,i,iPO4_)) THEN
                FV1=(Bio(k,i,iPO4_)-VSMALL)/FV2
                DO iphy=1,Nphy
                  NupPO4(k,i,iphy)=NupPO4(k,i,iphy)*FV1
                END DO
                DO ibac=1,Nbac
                  NupPO4_ba(k,i,ibac)=NupPO4_ba(k,i,ibac)*FV1
                END DO
                NtoPBAC(k,i)=NtoPBAC(k,i)*FV1
              END IF
!

! MFR - Only if Fe is being calculated

              IF(IRON==1)THEN
               FV2=totFe_d(k,i)*dtbio
               IF (FV2.gt.Bio(k,i,iFeO_)) THEN
                 FV1=(Bio(k,i,iFeO_)-VSMALL)/FV2
                 DO iphy=1,Nphy
                   NupFe(k,i,iphy)=NupFe(k,i,iphy)*FV1
                 END DO
                 DO ibac=1,Nbac
                   NupFe_ba(k,i,ibac)=NupFe_ba(k,i,ibac)*FV1
                 END DO
                 NtoFeBAC(k,i)=NtoFeBAC(k,i)*FV1
               END IF
              END IF  !IRON
!
!  Bacteria are the only group to take up DOC.  Remove BAC DON and
!  BAC DOP uptake from total uptake; adjust uptake and add back.
!
              FV2=totDOC_d(k,i)*dtbio
              IF (FV2.gt.Bio(k,i,iDOMC(ilab))) THEN
                FV1=(Bio(k,i,iDOMC(ilab))-VSMALL)/FV2
                totDOC_d(k,i)=totDOC_d(k,i)*FV1
                DO ibac=1,Nbac
                  NupDOC_ba(k,i,ibac)=NupDOC_ba(k,i,ibac)*FV1
                  totDON_d(k,i)=totDON_d(k,i)-NupDON_ba(k,i,ibac)
                  NupDON_ba(k,i,ibac)=NupDON_ba(k,i,ibac)*FV1
                  totDON_d(k,i)=totDON_d(k,i)+NupDON_ba(k,i,ibac)
                  totDOP_d(k,i)=totDOP_d(k,i)-NupDOP_ba(k,i,ibac)
                  NupDOP_ba(k,i,ibac)=NupDOP_ba(k,i,ibac)*FV1
                  totDOP_d(k,i)=totDOP_d(k,i)+NupDOP_ba(k,i,ibac)
                END DO
              END IF
!
!  Remove BAC DON uptake from total uptake; adjust uptake and add back.
!
              FV2=totDON_d(k,i)*dtbio
              IF (FV2.gt.Bio(k,i,iDOMN(ilab))) THEN
                FV1=(Bio(k,i,iDOMN(ilab))-VSMALL)/FV2
                totDON_d(k,i)=totDON_d(k,i)*FV1
                totDOC_d(k,i)=totDOC_d(k,i)*FV1
                DO iphy=1,Nphy
                  NupDON(k,i,iphy)=NupDON(k,i,iphy)*FV1
                END DO
                DO ibac=1,Nbac
                  NupDON_ba(k,i,ibac)=NupDON_ba(k,i,ibac)*FV1
                  NupDOC_ba(k,i,ibac)=NupDOC_ba(k,i,ibac)*FV1
                  totDOP_d(k,i)=totDOP_d(k,i)-NupDOP_ba(k,i,ibac)
                  NupDOP_ba(k,i,ibac)=NupDOP_ba(k,i,ibac)*FV1
                  totDOP_d(k,i)=totDOP_d(k,i)+NupDOP_ba(k,i,ibac)
                END DO
              END IF
!
!  Remove BAC DOP uptake from total uptake; adjust uptake and add back.
!
              FV2=totDOP_d(k,i)*dtbio
              IF (FV2.gt.Bio(k,i,iDOMP(ilab))) THEN
                FV1=(Bio(k,i,iDOMP(ilab))-VSMALL)/FV2
                totDOP_d(k,i)=totDOP_d(k,i)*FV1
                totDOC_d(k,i)=totDOC_d(k,i)*FV1
                DO iphy=1,Nphy
                  NupDOP(k,i,iphy)=NupDOP(k,i,iphy)*FV1
                END DO
                DO ibac=1,Nbac
                  NupDOP_ba(k,i,ibac)=NupDOP_ba(k,i,ibac)*FV1
                  totDON_d(k,i)=totDON_d(k,i)-NupDON_ba(k,i,ibac)
                  NupDON_ba(k,i,ibac)=NupDON_ba(k,i,ibac)*FV1
                  totDON_d(k,i)=totDON_d(k,i)+NupDON_ba(k,i,ibac)
                  NupDOC_ba(k,i,ibac)=NupDOC_ba(k,i,ibac)*FV1
                END DO
              END IF
            END DO
          END DO
!
!  Increase particulate nutrients by the amount of the uptake.
!
          DO iphy=1,Nphy
            DO i=1,nea
              if(idry_e(i)==1) cycle
              DO k=kbe(i)+1,nvrt
                Bio_new(k,i,iPhyN(iphy))=Bio_new(k,i,iPhyN(iphy))+      &
     &                                   NupNO3(k,i,iphy)+              &
     &                                   NupNH4(k,i,iphy)+              &
     &                                   NupDON(k,i,iphy)
                Bio_new(k,i,iPhyP(iphy))=Bio_new(k,i,iPhyP(iphy))+      &
     &                                   NupPO4(k,i,iphy)+              &
     &                                   NupDOP(k,i,iphy)
                IF(IRON==1) Bio_new(k,i,iPhyF(iphy))=                   &
     &                                   Bio_new(k,i,iPhyF(iphy))+      &
     &                                   NupFe(k,i,iphy)
                IF (iPhyS(iphy).gt.0) THEN
                  Bio_new(k,i,iPhyS(iphy))=Bio_new(k,i,iPhyS(iphy))+    &
     &                                     NupSiO(k,i,iphy)
                END IF
!
!  Update nutrient arrays for growth and budgets. Bacterial uptake
!  included below.
!
                Bio_new(k,i,iNO3_)=Bio_new(k,i,iNO3_)-                  &
     &                             NupNO3(k,i,iphy)
                Bio_new(k,i,iNH4_)=Bio_new(k,i,iNH4_)-                  &
     &                             NupNH4(k,i,iphy)
                Bio_new(k,i,iSiO_)=Bio_new(k,i,iSiO_)-                  &
     &                             NupSiO(k,i,iphy)
                Bio_new(k,i,iPO4_)=Bio_new(k,i,iPO4_)-                  &
     &                             NupPO4(k,i,iphy)
                IF(IRON==1) Bio_new(k,i,iFeO_)=                         &
     &                             Bio_new(k,i,iFeO_)-                  &
     &                             NupFe (k,i,iphy)
                Bio_new(k,i,iDOMN(ilab))=Bio_new(k,i,iDOMN(ilab))-      &
     &                                   NupDON(k,i,iphy)
                Bio_new(k,i,iDOMP(ilab))=Bio_new(k,i,iDOMP(ilab))-      &
     &                                   NupDOP(k,i,iphy)
              END DO
            END DO
          END DO
!
!  Nitrification fixes DIC into DOC.
!
          DO i=1,nea
            if(idry_e(i)==1) cycle
            DO k=kbe(i)+1,nvrt           
              Bio_new(k,i,iDIC_)=Bio_new(k,i,iDIC_)-                    &
     &                           NitrBAC(k,i)
            END DO
          END DO
!
!  Add nitrifying bacteria biomass to heterotrophic bacteria biomass.
!  Adding PON, POP, POFe to BacC arrays at current C2_BAC ratios.
!
          DO ibac=1,Nbac
            DO i=1,nea
              if(idry_e(i)==1) cycle
              DO k=kbe(i)+1,nvrt
                Bio_new(k,i,iBacC(ibac))=Bio_new(k,i,iBacC(ibac))+      &
     &                                   NitrBAC(k,i)
                Bio_new(k,i,iBacN(ibac))=Bio_new(k,i,iBacN(ibac))+      &
     &                                   NtoNBAC(k,i)
                Bio_new(k,i,iBacP(ibac))=Bio_new(k,i,iBacP(ibac))+      &
     &                                   NtoPBAC(k,i)
                IF(IRON==1) Bio_new(k,i,iBacF(ibac))=                   &
     &                                   Bio_new(k,i,iBacF(ibac))+      &
     &                                   NtoFeBAC(k,i)
              END DO
            END DO
          END DO
!
!  Update nutrient arrays for nitrification.
!
          DO i=1,nea
            if(idry_e(i)==1) cycle
            DO k=kbe(i)+1,nvrt            
              Bio_new(k,i,iNO3_)=Bio_new(k,i,iNO3_)+                    &
     &                           NH4toNO3(k,i)
              Bio_new(k,i,iNH4_)=Bio_new(k,i,iNH4_)-                    &
     &                           (NH4toNO3(k,i)+NtoNBAC(k,i))
              Bio_new(k,i,iPO4_)=Bio_new(k,i,iPO4_)-                    &
     &                           NtoPBAC(k,i)
              IF(IRON==1) Bio_new(k,i,iFeO_)=Bio_new(k,i,iFeO_)-        &
     &                           NtoFeBAC(k,i)
            END DO
          END DO
!
!-----------------------------------------------------------------------
!  Light mediated carbon growth.
!-----------------------------------------------------------------------
!
!         May use rank specific outputs
          if(myrank==0) WRITE(600,*)'Keuphotic',Keuphotic(2161)
          DO i=1,nea
            if(idry_e(i)==1) cycle
            DO k=nvrt,Keuphotic(i),-1
              DO iphy=1,Nphy
                IF (Bio(k,i,iPhyC(iphy)).gt.0.0_r8) THEN
!
!  Calculate weighted average spectral absorption.
!
                  aPHYN_wa=0.0_r8
                  DO iband=1,NBands
                    aPHYN_wa=aPHYN_wa+(aPHYN_al(k,i,iphy,iband)*        &
     &                                 specir_scal(k,i,iband))
                  END DO
!
!  If Keuphotic(i) < N+1, and E0_nz(k,i)=0, this will cause pigments to
!  blow up. This should never happen, unless Keuphotic is not calcuated
!  properly. WPB
!
                  aPHYN_wa=aPHYN_wa/E0_nz(k,i)
!
!  Calculate "alfa" for HTAN function of P vs. I.
!  (conversion:  Ein/microEin * 10e3)
!
                  alfa(k,iphy)=(aPHYN_wa/Bio(k,i,iPhyC(iphy)))*         &
     &                          qu_yld(iphy)*0.001_r8
!
!  Light limited growth rate.
!
                  FV1=MAX(0.0_r8,E0_nz(k,i)-E0_comp(iphy))
                  FV2=E0_nz(k,i)-E0_inhib(iphy)
                  IF (FV2.gt.0.0_r8) THEN
                    Gt_ll(k,iphy)=GtALG(k,i,iphy)*                      &
     &                            TANH(alfa(k,iphy)*FV1/                &
     &                            GtALG(k,i,iphy))*                     &
     &                            EXP(-inhib_fac(iphy)*FV2)
                  ELSE
                    Gt_ll(k,iphy)=GtALG(k,i,iphy)*                      &
     &                            TANH(alfa(k,iphy)*FV1/                &
     &                            GtALG(k,i,iphy))
                  END IF
!
!  Nutrient limited growth rates.
!
!  REMEMBER that sinking speed to be set by gradient of limiting
!       nutrient, allowing for negative sinking. Try storing growth
!       rate terms in an array and using MAXLOC for if test.
!
!  Nitrogen limited growth rate.
!
                  IF (Bio(k,i,iPhyN(iphy)).gt.0.0_r8) THEN
                    FV1=Bio(k,i,iPhyC(iphy))/                           &
     &                  (Bio(k,i,iPhyN(iphy))+Bio_new(k,i,iPhyN(iphy)))
                    Gt_nl(k,iphy)=mu_bar_n(k,i,iphy)*                   &
     &                            (1.0_r8-ImaxC2nALG(iphy)*FV1)
                    Gt_nl(k,iphy)=MAX(0.0_r8,                           &
     &                                MIN(Gt_nl(k,iphy),                &
     &                                    GtALG(k,i,iphy)))
                  END IF
!
!  Silica limited growth rate.
!  Testing for silica incorporation.
!
                  IF (iPhyS(iphy).gt.0) THEN
                    IF ((HsSiO(iphy).lt.LARGE).and.                  &
     &                  (Bio(k,i,iPhyS(iphy)).gt.0.0_r8)) THEN
                      FV1=Bio(k,i,iPhyC(iphy))/                         &
     &                    (Bio(k,i,iPhyS(iphy))+                        &
     &                     Bio_new(k,i,iPhyS(iphy)))
                      Gt_sl(k,iphy)=mu_bar_s(k,i,iphy)*                 &
     &                              (1.0_r8-ImaxC2SiALG(iphy)*FV1)
                      Gt_sl(k,iphy)=MAX(0.0_r8,                         &
     &                                  MIN(Gt_sl(k,iphy),              &
     &                                      GtALG(k,i,iphy)))
                    ELSE
                      Gt_sl(k,iphy)=LARGE
                    END IF
                  ELSE
                    Gt_sl(k,iphy)=LARGE
                  END IF
!
!  Phosphorus limited growth rate.
!
                  IF ((HsPO4(iphy).lt.LARGE).and.                    &
     &                (Bio(k,i,iPhyP(iphy)).gt.0.0_r8)) THEN
                    FV1=Bio(k,i,iPhyC(iphy))/                           &
     &                  (Bio(k,i,iPhyP(iphy))+Bio_new(k,i,iPhyP(iphy)))
                    Gt_pl(k,iphy)=mu_bar_p(k,i,iphy)*                   &
     &                            (1.0_r8-ImaxC2pALG(iphy)*FV1)
                    Gt_pl(k,iphy)=MAX(0.0_r8,                           &
     &                                MIN(Gt_pl(k,iphy),                &
     &                                    GtALG(k,i,iphy)))
                  ELSE
                    Gt_pl(k,iphy)=LARGE
                  END IF
!
!  MFR - Only if iron is being calculated...

                  IF(IRON==1)THEN
!
!  Iron limited growth rate
!
                   IF ((HsFe(iphy).lt.LARGE).and.                     &
     &                 (Bio(k,i,iPhyF(iphy)).gt.0.0_r8)) THEN
                     FV1=Bio(k,i,iPhyC(iphy))/                           &
     &                   (Bio(k,i,iPhyF(iphy))+Bio_new(k,i,iPhyF(iphy)))
                     Gt_fl(k,iphy)=mu_bar_f(k,i,iphy)*                   &
     &                             (1.0_r8-ImaxC2FeALG(iphy)*FV1)
                     Gt_fl(k,iphy)=MAX(0.0_r8,                           &
     &                                 MIN(Gt_fl(k,iphy),                &
     &                                     GtALG(k,i,iphy)))
                   ELSE
                     Gt_fl(k,iphy)=LARGE
                   END IF

                  ELSE 
                    Gt_fl(k,iphy)=LARGE
                  END IF  !IRON

!
!  Realized growth rate is minimum of light or nutrient limited rate.
!
                  GtALG_r(k,i,iphy)=MIN(Gt_ll(k,iphy),Gt_nl(k,iphy),    &
     &                                  Gt_sl(k,iphy),Gt_pl(k,iphy),    &
     &                                  Gt_fl(k,iphy))
                  IF (GtALG_r(k,i,iphy).ge.LARGE) THEN
                    GtALG_r(k,i,iphy)=0.0_r8
                  END IF
!
!  Carbon growth calculations.
!
                  FV1=Bio(k,i,iPhyC(iphy))*GtALG_r(k,i,iphy)
                  Bio_new(k,i,iPhyC(iphy))=Bio_new(k,i,iPhyC(iphy))+    &
     &                                     FV1
                  Bio_new(k,i,iDIC_)=Bio_new(k,i,iDIC_)-                &
     &                               FV1

!
!  Pigment growth calculations.
!
 		  DO ipig=1,Npig
		    IF (iPigs(iphy,ipig).gt.0) THEN
                      itrc=iPigs(iphy,ipig)
                        IF (Bio(k,i,iPhyC(iphy)).gt.0.0_r8) THEN
                          FV1=Bio(k,i,itrc)*GtALG_r(k,i,iphy)
                          Bio_new(k,i,itrc)=Bio_new(k,i,itrc)+FV1
                        END IF
		      END IF
		    END DO
                  END IF
                END DO
              END DO
            END DO
!
!-----------------------------------------------------------------------
!  Bacterioplankton carbon growth terms.
!-----------------------------------------------------------------------
!
          DO i=1,nea
            if(idry_e(i)==1) cycle
            DO k=kbe(i)+1,nvrt
              Het_BAC=0.0_r8
              RelDOC1=0.0_r8
              RelDON1=0.0_r8
              RelDOP1=0.0_r8
              RelFe=0.0_r8
!
!  NOTE: Only DOC2/DON2 formation is in this section.
!        Take colored excretion off the top. 03/18/00
!        also, not excreting any DOP or Fe
!  REMEMBER, if excreting DOP and Fe, must address changes in growth if
!        tests. (see DON equations). 03/21/00.
!

              DO ibac=1,Nbac

!  MFR - Only if recalcitrant organic matter is being calculated...
                IF(Ndom==2)THEN

                 FV1=NupDOC_ba(k,i,ibac)*ExBAC_c*                    &
     &               (1.0_r8-cDOCfrac_c(irct))
                 FV2=NupDOC_ba(k,i,ibac)*ExBAC_c*                    &
     &               cDOCfrac_c(irct)
                 FV3=NupDON_ba(k,i,ibac)*ExBAC_n
                           
                 Bio_new(k,i,iDOMC(irct))=Bio_new(k,i,iDOMC(irct))+      &
     &                                    FV1
                 Bio_new(k,i,iCDMC(irct))=Bio_new(k,i,iCDMC(irct))+      &
     &                                    FV2
                 Bio_new(k,i,iDOMN(irct))=Bio_new(k,i,iDOMN(irct))+      &
     &                                    FV3
                
!
!  As we are taking it off the top, must remove from DOMN1 now. No other
!  organisms use DOMC1, so net term (totDOC_d) can be used in budgeting
!  below. This saves cycles, but makes code difficult to read. WPB
!
                 Bio_new(k,i,iDOMN(ilab))=Bio_new(k,i,iDOMN(ilab))-      &
     &                                   FV3
!
!  Remove from uptake.
!
                 NupDOC_ba(k,i,ibac)=NupDOC_ba(k,i,ibac)-                &
     &                              (FV1+FV2)
                 NupDON_ba(k,i,ibac)=NupDON_ba(k,i,ibac)-                &
     &                              FV3

                END IF  !Ndom
!
!  Determine growth limitation. Assuming 100% efficiency for N, P, Fe.
!  If DOMC=0, or DOMN=0, or DOMP=0, then NupDOC_ba = NupDON_ba =
!  NupDOP_ba = 0 and none of the divisions below are accessed. WPB
!
                Bac_G(1)=NupDOC_ba(k,i,ibac)*Bac_Ceff
                Bac_G(2)=(NupDON_ba(k,i,ibac)+                          &
     &                    NupNH4_ba(k,i,ibac))*                         &
     &                   C2nBAC
                Bac_G(3)=(NupDOP_ba(k,i,ibac)+                          &
     &                    NupPO4_ba(k,i,ibac))*                         &
     &                   C2pBAC

                IF(IRON==1)THEN
                  Bac_G(4)=NupFe_ba(k,i,ibac)*C2FeBAC
                ELSE 
!                 Bac_G(4)= LARGE
                ENDIF
!  Energy limited case. All excess nutrients returned in inorganic form.
!
                IF ((Bac_G(1).le.Bac_G(2)).and.                         &
     &              (Bac_G(1).le.Bac_G(3)).and.                         &
     &              (Bac_G(1).le.Bac_G(4))) THEN
                  Het_BAC=Bac_G(1)
                  FV1=Bac_G(1)*N2cBAC
                  FV2=Bac_G(1)*P2cBAC
                  FV3=Bac_G(1)*Fe2cBAC
                  Bio_new(k,i,iBacN(ibac))=Bio_new(k,i,iBacN(ibac))+    &
     &                                     FV1
                  Bio_new(k,i,iBacP(ibac))=Bio_new(k,i,iBacP(ibac))+    &
     &                                     FV2
                  IF(IRON==1) Bio_new(k,i,iBacF(ibac))=                 &
     &                                     Bio_new(k,i,iBacF(ibac))+    &
     &                                     FV3
!
!  Uptake arrays should probably now be negative. If NH4 or PO4 is
!  positive, then there is some uptake of inorganic forms, but this
!  value will be less than the original Nup value because of IF test.
!
                  NupNH4_ba(k,i,ibac)=FV1-NupDON_ba(k,i,ibac)
                  NupPO4_ba(k,i,ibac)=FV2-NupDOP_ba(k,i,ibac)
!
!  Because Fe is considered to be all inorganic, only net uptake of Fe
!  is needed.
!
                  RelFe=NupFe_ba(k,i,ibac)-FV3
                  NupFe_ba(k,i,ibac)=FV3
!
!  Nitrogen limited case. Excess nutrients returned in organic form
!  first, inorganic second.
!
                ELSE IF ((Bac_G(2).le.Bac_G(3)).and.                    &
     &                   (Bac_G(2).le.Bac_G(4))) THEN
                  Het_BAC=Bac_G(2)
                  FV2=Bac_G(2)*P2cBAC
                  FV3=Bac_G(2)*Fe2cBAC
                  Bio_new(k,i,iBacN(ibac))=Bio_new(k,i,iBacN(ibac))+    &
     &                                     (NupDON_ba(k,i,ibac)+        &
     &                                      NupNH4_ba(k,i,ibac))
                  Bio_new(k,i,iBacP(ibac))=Bio_new(k,i,iBacP(ibac))+    &
     &                                     FV2
                  IF(IRON==1) Bio_new(k,i,iBacF(ibac))=                 &
     &                                     Bio_new(k,i,iBacF(ibac))+    &
     &                                     FV3
!
!  Uptake arrays will now reflect release of inorganic and organic
!  revision of uptake.
!
                  FV1=(Bac_G(1)-Bac_G(2))*I_Bac_Ceff
                  NupDOC_ba(k,i,ibac)=NupDOC_ba(k,i,ibac)-FV1
                  RelDOC1=FV1
!
!  To get accurate DOP from C2pDOC, must add back excreted DOC.
!
                  FV4=FV1*R_ExBAC_c*                                &
!!   &                DOC_frac(k,i)*                                    &
     &                Bio(k,i,iDOMP(ilab))/                             &
     &                Bio(k,i,iDOMC(ilab))
                  FV5=FV2-(NupDOP_ba(k,i,ibac)+                         &
                           NupPO4_ba(k,i,ibac)-FV4)
!
!  If FV5 is positive then released DOP is required for bacteria growth.
!
                  IF (FV5.lt.0.0_r8) THEN
                    RelDOP1=FV4
                    NupPO4_ba(k,i,ibac)=NupPO4_ba(k,i,ibac)+FV5
                  ELSE
                    RelDOP1=FV4-FV5
                  END IF
                  NupDOP_ba(k,i,ibac)=NupDOP_ba(k,i,ibac)-RelDOP1
!
!  Release Fe.
!
                  RelFe=NupFe_ba(k,i,ibac)-FV3
                  NupFe_ba(k,i,ibac)=FV3
!
!  Phosphorous limited case. Excess nutrients returned in organic form
!  first, inorganic second.
!
                ELSE IF (Bac_G(3).le.Bac_G(4)) THEN
                  Het_BAC=Bac_G(3)
                  FV2=Bac_G(3)*N2cBAC
                  FV3=Bac_G(3)*Fe2cBAC
                  Bio_new(k,i,iBacN(ibac))=Bio_new(k,i,iBacN(ibac))+    &
     &                                     FV2
                  Bio_new(k,i,iBacP(ibac))=Bio_new(k,i,iBacP(ibac))+    &
     &                                     (NupDOP_ba(k,i,ibac)+        &
     &                                      NupPO4_ba(k,i,ibac))
                  IF(IRON==1) Bio_new(k,i,iBacF(ibac))=                 &
     &                                     Bio_new(k,i,iBacF(ibac))+    &
     &                                     FV3
!
!  Uptake arrays will now reflect release of inorganic and organic
!  revision of uptake.
!
                  FV1=(Bac_G(1)-Bac_G(3))*I_Bac_Ceff
                  NupDOC_ba(k,i,ibac)=NupDOC_ba(k,i,ibac)-FV1
                  RelDOC1=FV1
!
!  To get accurate DON from C2nDOC, must add back excreted DOC.
!
                  FV4=FV1*R_ExBAC_c*                                &
!!   &                DOC_frac(k,i)*                                    &
     &                (Bio(k,i,iDOMN(ilab))/                            &
     &                 Bio(k,i,iDOMC(ilab)))*Frac_ExBAC_n
                  FV5=FV2-(NupDON_ba(k,i,ibac)+                         &
     &                     NupNH4_ba(k,i,ibac)-FV4)
!
!  If FV5 is positive then released DON is required for bacteria growth.
!
                  IF (FV5.lt.0.0_r8) THEN
                    RelDON1=FV4
                    NupNH4_ba(k,i,ibac)=NupNH4_ba(k,i,ibac)+FV5
                  ELSE
                    RelDON1=FV4-FV5
                  END IF
                  NupDON_ba(k,i,ibac)=NupDON_ba(k,i,ibac)-RelDON1
!
!  Release Fe.
!
                  RelFe=NupFe_ba(k,i,ibac)-FV3
                  NupFe_ba(k,i,ibac)=FV3
!
!  Fe limited case. Excess nutrients returned in organic form
!  first, inorganic second.
!
                ELSE
                  Het_BAC=Bac_G(4)
                  FV2=Bac_G(4)*N2cBAC
                  FV3=Bac_G(4)*P2cBAC
                  Bio_new(k,i,iBacN(ibac))=Bio_new(k,i,iBacN(ibac))+    &
     &                                     FV2
                  Bio_new(k,i,iBacP(ibac))=Bio_new(k,i,iBacP(ibac))+    &
     &                                     FV3
                  IF(IRON==1) Bio_new(k,i,iBacF(ibac))=                 &
     &                                     Bio_new(k,i,iBacF(ibac))+    &
     &                                     NupFe_ba(k,i,ibac)
!
!  Uptake arrays will now reflect release of inorganic and organic
!  revision of uptake.
!
                  FV1=(Bac_G(1)-Bac_G(4))*I_Bac_Ceff
                  NupDOC_ba(k,i,ibac)=NupDOC_ba(k,i,ibac)-FV1
                  RelDOC1=FV1
!
!  To get accurate DON from C2nDOC, must add back excreted DOC.
!
                  FV4=FV1*R_ExBAC_c*                                &
!!   &                DOC_frac(k,i)*                                    &
     &                Bio(k,i,iDOMN(ilab))/                             &
     &                Bio(k,i,iDOMC(ilab))*Frac_ExBAC_n
                  FV5=FV2-(NupDON_ba(k,i,ibac)+                         &
     &                     NupNH4_ba(k,i,ibac)-FV4)
!
!  If FV5 is positive then released DON is required for bacteria growth.
!
                  IF (FV5.lt.0.0_r8) THEN
                    RelDON1=FV4
                    NupNH4_ba(k,i,ibac)=NupNH4_ba(k,i,ibac)+FV5
                  ELSE
                    RelDON1=FV4-FV5
                  END IF
                  NupDON_ba(k,i,ibac)=NupDON_ba(k,i,ibac)-RelDON1
!
!  To get accurate DOP from C2pDOC, must add back excreted DOC.
!
                  FV4=FV1*R_ExBAC_c*                                &
!!   &               DOC_frac(k,i)*                                     &
     &                Bio(k,i,iDOMP(ilab))/                             &
     &                Bio(k,i,iDOMC(ilab))
                  FV5=FV2-(NupDOP_ba(k,i,ibac)+                         &
     &                     NupPO4_ba(k,i,ibac)-FV4)
!
!  If FV5 is positive then released DOP is required for bacteria growth.
!
                  IF (FV5.lt.0.0_r8) THEN
                    RelDOP1=FV4
                    NupPO4_ba(k,i,ibac)=NupPO4_ba(k,i,ibac)+FV5
                  ELSE
                    RelDOP1=FV4-FV5
                  END IF
                  NupDOP_ba(k,i,ibac)=NupDOP_ba(k,i,ibac)-RelDOP1
                END IF
!
!  Increment nutrient arrays.
!
                Bio_new(k,i,iBacC(ibac))=Bio_new(k,i,iBacC(ibac))+      &
     &                                   Het_BAC
                FV1=NupDOC_ba(k,i,ibac)-Het_BAC
                Bio_new(k,i,iDIC_)=Bio_new(k,i,iDIC_)+                  &
     &                             FV1

!
!  NOTE: to be strictly accurate we should remove RelDOC1 from DOCNP1,
!       and then add it back, since NupDOC_ba is a net term. This should
!       wash out in the budgeting.
!
                Bio_new(k,i,iDOMC(ilab))=Bio_new(k,i,iDOMC(ilab))-      &
     &                                   (totDOC_d(k,i)-RelDOC1)
!!   &                                   (totDOC_d(k,i)-RelDOC1)*       &
!!   &                                   DOC_frac(k,i)
!!              IF(CDOC==1) Bio_new(k,i,iCDMC(ilab))=                   &
!!   &                                   Bio_new(k,i,iCDMC(ilab))-      &
!!   &                                   (totDOC_d(k,i)-RelDOC1)*       &
!!   &                                   (1.0_r8-DOC_frac(k,i))
!!
!  This is inclusive of RelDOX1, excretion of DON1 removed above.
!
                Bio_new(k,i,iDOMN(ilab))=Bio_new(k,i,iDOMN(ilab))-      &
     &                                   NupDON_ba(k,i,ibac)
                Bio_new(k,i,iDOMP(ilab))=Bio_new(k,i,iDOMP(ilab))-      &
     &                                   NupDOP_ba(k,i,ibac)
                Bio_new(k,i,iNH4_)=Bio_new(k,i,iNH4_)-                  &
     &                             NupNH4_ba(k,i,ibac)
                Bio_new(k,i,iPO4_)=Bio_new(k,i,iPO4_)-                  &
     &                             NupPO4_ba(k,i,ibac)
                IF(IRON==1) Bio_new(k,i,iFeO_)=Bio_new(k,i,iFeO_)-      &
     &                             NupFe_ba(k,i,ibac)
              END DO
            END DO
          END DO


! Marta Rodrigues
!-----------------------------------------------------------------------
!  Zooplankton growth terms
!-----------------------------------------------------------------------
!
          DO izoo=1,Nzoo
            DO i=1,nea
              if(idry_e(i)==1) cycle
	      DO k=kbe(i)+1, nvrt	      
!
!  Carbon growth calculations
!                 	      
	        FV1=Bio(k,i,iZooC(izoo))*Gt_z(k,i,izoo)
		Bio_new(k,i,iZooC(izoo))=Bio_new(k,i,iZooC(izoo))+      &
		                         FV1

!
!  Nitrogen growth calculations
!
		FV2=foodN(k,i,izoo)/foodC(k,i,izoo)
		Bio_new(k,i,iZooN(izoo))=Bio_new(k,i,iZooN(izoo))+      &
		                         FV1*FV2

!
!  Phosphorus growth calculations
!
		FV3=foodP(k,i,izoo)/foodC(k,i,izoo)
		Bio_new(k,i,iZooP(izoo))=Bio_new(k,i,iZooP(izoo))+      &
		                         FV1*FV3
	      END DO
	    END DO
	  END DO		 	 

!-----------------------------------------------------------------------

!
!-----------------------------------------------------------------------
!  Phytoplankton Losses.
!-----------------------------------------------------------------------
!

          DO iphy=1,Nphy
            DO i=1,nea
              if(idry_e(i)==1) cycle
              DO k=kbe(i)+1,nvrt
!
!  Excretion.
!
                IF ((C2nALG(k,i,iphy).ge.                               &
     &               C2nALGminABS(iphy)).and.                        &
     &              (C2pALG(k,i,iphy).ge.                               &
     &               C2pALGminABS(iphy)).and.                        &
     &              (HsSiO(iphy).gt.LARGE)) THEN
                  FV1=Bio(k,i,iPhyC(iphy))*ExALG(iphy)
                  Bio_new(k,i,iPhyC(iphy))=Bio_new(k,i,iPhyC(iphy))-    &
     &                                     FV1

!
!  No excretion of CDOC.
!
                  Bio_new(k,i,iDOMC(ilab))=Bio_new(k,i,iDOMC(ilab))+    &
     &                                     FV1
                ELSE IF ((C2nALG(k,i,iphy).ge.                          &
     &                    C2nALGminABS(iphy)).and.                   &
     &                   (C2pALG(k,i,iphy).ge.                          &
     &                    C2pALGminABS(iphy)).and.                   &
     &                   (C2sALG(k,i,iphy).ge.                          &
     &                    C2SiALGminABS(iphy))) THEN
                  FV1=Bio(k,i,iPhyC(iphy))*ExALG(iphy)
                  Bio_new(k,i,iPhyC(iphy))=Bio_new(k,i,iPhyC(iphy))-    &
     &                                     FV1

!
!  No excretion of CDOC.
!
                  Bio_new(k,i,iDOMC(ilab))=Bio_new(k,i,iDOMC(ilab))+    &
     &                                     FV1
                END IF
!
!  Grazing.
!  And lises (Marta Rodrigues)

                IF (Bio(k,i,iPhyC(iphy)).gt.refuge(k,i,iphy)) THEN
!
!  Carbon calculations.
!
! Marta Rodrigues
!  Changed to account for zooplankton

! Calculate grazing of phytoplankton due to zooplankton (carbon)
	          DO izoo=1,Nzoo  
		     FV1=Gt_zoo_phy(k,i,izoo,iphy)*Bio(k,i,izooC(izoo))
		     !graz_act(k,i,iphy)=graz_act(k,i,iphy)*FV1
                     graz_act(k,i,iphy)=FV1
		  END DO 

!MFR          FV1=graz_act(k,i,iphy)*Bio(k,i,iPhyC(iphy))  
                      
		  FV1=HsGRZ(iphy)*Bio(k,i,iphyC(iphy))
                  FV2=HsGRZ(iphy)*Bio(k,i,iphyC(iphy))+graz_act(k,i,iphy)
               
                  Bio_new(k,i,iPhyC(iphy))=Bio_new(k,i,iPhyC(iphy))-    &
     &                                     FV2

                  Bio_new(k,i,iFecC(isfc))=Bio_new(k,i,iFecC(isfc))+    &
     &                                     FecPEL(iphy,isfc)*FV1
                  IF(Nfec==2) Bio_new(k,i,iFecC(iffc))=                 &
     &                                     Bio_new(k,i,iFecC(iffc))+    &
     &                                     FecPEL(iphy,iffc)*FV1
                  FV3=FecDOC(iphy)*FV1
                  Bio_new(k,i,iDOMC(ilab))=Bio_new(k,i,iDOMC(ilab))+    &
     &                                     (1.0_r8-cDOCfrac_c(ilab))*&
     &                                     FV3
                  IF(CDOC==1) Bio_new(k,i,iCDMC(ilab))=                 &
     &                                     Bio_new(k,i,iCDMC(ilab))+    &
     &                                     cDOCfrac_c(ilab)*FV3
                  Bio_new(k,i,iDIC_)=Bio_new(k,i,iDIC_)+                &
     &                               FecCYC(iphy)*FV1


!
!  Nitrogen calculations.
!
!MFR          FV2=graz_act(k,i,iphy)*Bio(k,i,iPhyN(iphy))

                  IF(C2nALG(k,i,iphy).gt.0.d0)THEN
                   FV3=HsGRZ(iphy)*Bio(k,i,iphyN(iphy))+              &
		       graz_act(k,i,iphy)/C2nALG(k,i,iphy)
                  ELSE
                   FV3=HsGRZ(iphy)*Bio(k,i,iphyN(iphy)) 
                  ENDIF

                  FV2=HsGRZ(iphy)*Bio(k,i,iphyN(iphy))

                  Bio_new(k,i,iPhyN(iphy))=Bio_new(k,i,iPhyN(iphy))-    &
     &                                     FV3
                  Bio_new(k,i,iFecN(isfc))=Bio_new(k,i,iFecN(isfc))+    &
     &                                     FecPEL(iphy,isfc)*FV2
                  IF(Nfec==2) Bio_new(k,i,iFecN(iffc))=                 &
     &                                     Bio_new(k,i,iFecN(iffc))+    &
     &                                     FecPEL(iphy,iffc)*FV2
                  Bio_new(k,i,iDOMN(ilab))=Bio_new(k,i,iDOMN(ilab))+    &
     &                                     FecDOC(iphy)*FV2
                  Bio_new(k,i,iNH4_)=Bio_new(k,i,iNH4_)+                &
     &                               FecCYC(iphy)*FV2
!
!  Silica calculations.
!
                  IF (iPhyS(iphy).gt.0) THEN
!MFR            FV2=graz_act(k,i,iphy)*Bio(k,i,iPhyS(iphy))

                    IF(C2sALG(k,i,iphy).gt.0.d0)THEN 
                     FV3=HsGRZ(iphy)*Bio(k,i,iphyS(iphy))+            &
		         graz_act(k,i,iphy)/C2sALG(k,i,iphy)
                    ELSE
                     FV3=HsGRZ(iphy)*Bio(k,i,iphyS(iphy))
                    ENDIF

                    FV2=HsGRZ(iphy)*Bio(k,i,iphyS(iphy)) 

                    Bio_new(k,i,iPhyS(iphy))=Bio_new(k,i,iPhyS(iphy))-  &
     &                                       FV3
!
!  Assuming that the fraction of material lost via sloppy feeding/cell
!  lysis also results in silica tests being put into FecS pool.
!
                    Bio_new(k,i,iFecS(isfc))=Bio_new(k,i,iFecS(isfc))+  &
     &                                       FecDOC(iphy)*FV2
                    IF(Nfec==2) Bio_new(k,i,iFecS(iffc))=               &
     &                                       Bio_new(k,i,iFecS(iffc))+  &
     &                                       (1.0_r8-FecDOC(iphy))*     &
     &                                       FV2
                  END IF
!
!  Phosphorus calculations.
!
!MFR          FV2=graz_act(k,i,iphy)*Bio(k,i,iPhyP(iphy))

                  IF(C2pALG(k,i,iphy).gt.0.d0)THEN
                    FV3=HsGRZ(iphy)*Bio(k,i,iphyP(iphy))+            &
		        graz_act(k,i,iphy)/C2pALG(k,i,iphy)  
		  ELSE
                    FV3=HsGRZ(iphy)*Bio(k,i,iphyP(iphy))
                  ENDIF

                  FV2=HsGRZ(iphy)*Bio(k,i,iphyP(iphy))                  

                  Bio_new(k,i,iPhyP(iphy))=Bio_new(k,i,iPhyP(iphy))-    &
     &                                     FV3
                  Bio_new(k,i,iFecP(isfc))=Bio_new(k,i,iFecP(isfc))+    &
     &                                     FecPEL(iphy,isfc)*FV2
                  IF(Nfec==2) Bio_new(k,i,iFecP(iffc))=                 &
     &                                     Bio_new(k,i,iFecP(iffc))+    &
     &                                     FecPEL(iphy,iffc)*FV2
                  Bio_new(k,i,iDOMP(ilab))=Bio_new(k,i,iDOMP(ilab))+    &
     &                                     FecDOC(iphy)*FV2
                  Bio_new(k,i,iPO4_)=Bio_new(k,i,iPO4_)+                &
     &                               FecCYC(iphy)*FV2
!
! MFR - Only if Fe is being calculated

                  IF(IRON==1)THEN
!
!  Iron calculations. Assuming no DOMF.
!
!MFR          FV2=graz_act(k,i,iphy)*Bio(k,i,iPhyF(iphy))

                   IF(C2fALG(k,i,iphy).gt.0.d0)THEN
                     FV3=HsGRZ(iphy)*Bio(k,i,iphyF(iphy))+            &
		         graz_act(k,i,iphy)/C2fALG(k,i,iphy)
                   ELSE
                     FV3=HsGRZ(iphy)*Bio(k,i,iphyF(iphy))
                   ENDIF 

                   FV2=HsGRZ(iphy)*Bio(k,i,iphyF(iphy))
		  
                   Bio_new(k,i,iPhyF(iphy))=Bio_new(k,i,iPhyF(iphy))-    &
     &                                      FV3
                   Bio_new(k,i,iFecF(isfc))=Bio_new(k,i,iFecF(isfc))+    &
     &                                      FecPEL(iphy,isfc)*FV2
                   IF(Nfec==2) Bio_new(k,i,iFecF(iffc))=                 &
     &                                      Bio_new(k,i,iFecF(iffc))+    &
     &                                      FecPEL(iphy,iffc)*FV2
                   Bio_new(k,i,iFeO_)=Bio_new(k,i,iFeO_)+                &
     &                                   (FecCYC(iphy)+                  &
     &                                   FecDOC(iphy))*FV2

                  END IF  !IRON
                END IF  !refuge
              END DO
            END DO
          END DO
!
!  Pigment Grazing.  No fecal or dissolved terms for pigments.
!
          DO ipig=1,Npig
            DO iphy=1,Nphy
              IF (iPigs(iphy,ipig).gt.0) THEN
                itrc=iPigs(iphy,ipig)
                DO i=1,nea
                  if(idry_e(i)==1) cycle
                  DO k=kbe(i)+1,nvrt                  
                    IF (Bio(k,i,iPhyC(iphy)).gt.refuge(k,i,iphy)) THEN
                      FV1=HsGRZ(iphy)*Bio(k,i,itrc)+                 &
		          graz_act(k,i,iphy)*Bio(k,i,itrc)/Bio(k,i,iPhyC(iphy))  
		      Bio_new(k,i,itrc)=Bio_new(k,i,itrc) - FV1
                    END IF
                  END DO
                END DO
              END IF
            END DO
          END DO

! Marta Rodrigues (! To check later...)
! Sinking

        DO iphy=1,Nphy
          DO i=1,nea
            if(idry_e(i)==1) cycle
            DO k=kbe(i)+1,nvrt-1
            
              Bio_new(k,i,iPhyC(iphy))=Bio_new(k,i,iPhyC(iphy))+&
                                    -WS(iphy)*((Bio(k+1,i,iPhyC(iphy))- &
                                    Bio(k-1,i,iPhyC(iphy)))/(ze(k+1,i)-   &
                                    ze(k-1,i)))

              Bio_new(k,i,iPhyN(iphy))=Bio_new(k,i,iPhyN(iphy))&
                                    -WS(iphy)*((Bio(k+1,i,iPhyN(iphy))- &
                                    Bio(k-1,i,iPhyN(iphy)))/(ze(k+1,i)-   &
                                    ze(k-1,i)))

              Bio_new(k,i,iPhyP(iphy))=Bio_new(k,i,iPhyP(iphy))+&
                                    -WS(iphy)*((Bio(k+1,i,iPhyP(iphy))- &
                                    Bio(k-1,i,iPhyP(iphy)))/(ze(k+1,i)-   &
                                    ze(k-1,i)))

              IF(IRON==1)THEN
               Bio_new(k,i,iPhyF(iphy))=Bio_new(k,i,iPhyF(iphy))+&
                                    -WS(iphy)*((Bio(k+1,i,iPhyF(iphy))- &
                                    Bio(k-1,i,iPhyF(iphy)))/(ze(k+1,i)-   &
                                    ze(k-1,i)))
              END IF  !IRON

              IF(iphy<=2) THEN

                Bio_new(k,i,iPhyS(iphy))=Bio_new(k,i,iPhyS(iphy))+&
                                      -WS(iphy)*((Bio(k+1,i,iPhyS(iphy))- &
                                      Bio(k-1,i,iPhyS(iphy)))/(ze(k+1,i)-   &
                                      ze(k-1,i)))
              END IF

              DO ipig=1,Npig
                IF (iPigs(iphy,ipig).gt.0) THEN
                  itrc=iPigs(iphy,ipig)

                  Bio_new(k,i,itrc)=-WS(iphy)*((Bio(k+1,i,itrc)- &
                                    Bio(k-1,i,itrc))/(ze(k+1,i)-   &
                                    ze(k-1,i)))
                END IF
  
              END DO
            END DO 
          END DO 
        END DO ! Nphy

!
!-----------------------------------------------------------------------
!  Bacterial losses.
!-----------------------------------------------------------------------
!
!  NOTE: Bacterial growth is completely reminerialized.
!
          DO ibac=1,Nbac
            DO i=1,nea
              if(idry_e(i)==1) cycle
              DO k=kbe(i)+1,nvrt             
!
!  Grazing calculation. (All fecal material to slow sinking pool.)
!
!! WPB - There appears to be some rounding errors that cause bacteria
!!       populations to drop just below initialization values.  Once
!!       they do, they never recover and the new lower values propagate
!!       through the model.  Only evident in Bac_P1 at the moment.
!!
!!                FV1=BacCYC(ng)*Bio_new(k,i,iBacC(ibac))
!!                FV2=BacPEL(ng)*Bio_new(k,i,iBacC(ibac))
!!                FV3=BacDOC(ng)*Bio_new(k,i,iBacC(ibac))
!!                FV4=FV1+FV2+FV3
!
!  Carbon calculations.
!
                Bio_new(k,i,iBacC(ibac))=Bio_new(k,i,iBacC(ibac))-      &
!!   &                                   FV4
     &                                   Bio_new(k,i,iBacC(ibac))
                Bio_new(k,i,iFecC(isfc))=Bio_new(k,i,iFecC(isfc))+      &
!!   &                                   FV2
     &                                   Bio_new(k,i,iBacC(ibac))*      &
     &                                   BacPEL
                Bio_new(k,i,iDOMC(ilab))=Bio_new(k,i,iDOMC(ilab))+      &
     &                                   (1.0_r8-cDOCfrac_c(ilab))*  &
!!   &                                   FV3
     &                                   Bio_new(k,i,iBacC(ibac))*      &
     &                                   BacDOC
                IF(CDOC==1) Bio_new(k,i,iCDMC(ilab))=                   &
     &                                   Bio_new(k,i,iCDMC(ilab))+      &
     &                                   cDOCfrac_c(ilab)*           &
!!   &                                   FV3
     &                                   Bio_new(k,i,iBacC(ibac))*      &
     &                                   BacDOC
                Bio_new(k,i,iDIC_)=Bio_new(k,i,iDIC_)+                  &
!!   &                             FV1
     &                             Bio_new(k,i,iBacC(ibac))*            &
     &                             BacCYC

!
!  Nitrogen calculations.
!
                Bio_new(k,i,iBacN(ibac))=Bio_new(k,i,iBacN(ibac))-      &
!!   &                                   N2cBAC(ng)*FV4
     &                                   Bio_new(k,i,iBacN(ibac))
                Bio_new(k,i,iFecN(isfc))=Bio_new(k,i,iFecN(isfc))+      &
!!   &                                   N2cBAC(ng)*FV2
     &                                   Bio_new(k,i,iBacN(ibac))*      &
     &                                   BacPEL
                Bio_new(k,i,iDOMN(ilab))=Bio_new(k,i,iDOMN(ilab))+      &
!!   &                                   N2cBAC(ng)*FV3
     &                                   Bio_new(k,i,iBacN(ibac))*      &
     &                                   BacDOC
                Bio_new(k,i,iNH4_)=Bio_new(k,i,iNH4_)+                  &
!!   &                             N2cBAC(ng)*FV1
     &                             Bio_new(k,i,iBacN(ibac))*            &
     &                             BacCYC
!
!  Phosphorous calculations.
!
                Bio_new(k,i,iBacP(ibac))=Bio_new(k,i,iBacP(ibac))-      &
!!   &                                   P2cBAC(ng)*FV4
     &                                   Bio_new(k,i,iBacP(ibac))
                Bio_new(k,i,iFecP(isfc))=Bio_new(k,i,iFecP(isfc))+      &
!!   &                                   P2cBAC(ng)*FV2
     &                                   Bio_new(k,i,iBacP(ibac))*      &
     &                                   BacPEL
                Bio_new(k,i,iDOMP(ilab))=Bio_new(k,i,iDOMP(ilab))+      &
!!   &                                   P2cBAC(ng)*FV3
     &                                   Bio_new(k,i,iBacP(ibac))*      &
     &                                   BacDOC
                Bio_new(k,i,iPO4_)=Bio_new(k,i,iPO4_)+                  &
!!   &                             P2cBAC(ng)*FV1
     &                             Bio_new(k,i,iBacP(ibac))*            &
     &                             BacCYC
!
! MFR - Only if Fe is being calculated...
                
                IF(IRON==1)THEN     

!  Iron calculations.
!
                 Bio_new(k,i,iBacF(ibac))=Bio_new(k,i,iBacF(ibac))-      &
!!   &                                    Fe2cBAC(ng)*FV4
     &                                    Bio_new(k,i,iBacF(ibac))
                 Bio_new(k,i,iFecF(isfc))=Bio_new(k,i,iFecF(isfc))+      &
!!   &                                    Fe2cBAC(ng)*FV2
     &                                    Bio_new(k,i,iBacF(ibac))*      &
     &                                    BacPEL
                 Bio_new(k,i,iFeO_)=Bio_new(k,i,iFeO_)+                  &
!!   &                              Fe2cBAC(ng)*(FV1+FV3)
     &                              Bio_new(k,i,iBacF(ibac))*            &
     &                              (BacDOC+BacCYC)

                END IF  !IRON                

              END DO
            END DO
          END DO


! Marta Rodrigues
!-----------------------------------------------------------------------
!  Zooplankton losses
!-----------------------------------------------------------------------

!  Excretion.
        
	DO izoo=1,Nzoo
          DO i=1,nea
            if(idry_e(i)==1) cycle
	    DO k=kbe(i)+1, nvrt
	    
!  Carbon calculations
	    
              FV1=Bio(k,i,iZooC(izoo))*ExZOO(izoo)
              Bio_new(k,i,iZooC(izoo))=Bio_new(k,i,iZooC(izoo))-    &
     &                                 FV1  

!  Excretion of DOC, CDOC and FecC.
!
              FV2=ZooDOC(izoo)*FV1    
	      Bio_new(k,i,iDOMC(ilab))=Bio_new(k,i,iDOMC(ilab))+    &
	                               (1.0_r8-cDOCfrac_c(ilab))*   &
     &                                 FV1 
              IF(CDOC==1) Bio_new(k,i,iCDMC(ilab))=                 &
     &                                 Bio_new(k,i,iCDMC(ilab))+    &
     &                                 cDOCfrac_c(ilab)*FV1     
              Bio_new(k,i,iFecC(isfc))=Bio_new(k,i,iFecC(isfc))+    &
     &                                 ZooPEL(izoo,isfc)*FV1
              IF(Nfec==2) Bio_new(k,i,iFecC(iffc))=                 &
     &                                 Bio_new(k,i,iFecC(iffc))+    &
     &                                 ZooPEL(izoo,iffc)*FV1
     
!  Nitrogen calculations

              FV1=Bio(k,i,iZooN(izoo))*ExZOO(izoo)
              Bio_new(k,i,iZooN(izoo))=Bio_new(k,i,iZooN(izoo))-    &
     &                                 FV1  

!  Excretion of DON, FecN, NH4

              Bio_new(k,i,iDOMN(ilab))=Bio_new(k,i,iDOMN(ilab))+    &
     &                                 ZooDOC(izoo)*FV1
              Bio_new(k,i,iFecN(isfc))=Bio_new(k,i,iFecN(isfc))+    &
     &                                 ZooPEL(izoo,isfc)*FV1
              IF(Nfec==2) Bio_new(k,i,iFecN(iffc))=                 &
     &                                 Bio_new(k,i,iFecN(iffc))+    &
     &                                 ZooPEL(izoo,iffc)*FV1
              Bio_new(k,i,iNH4_)=Bio_new(k,i,iNH4_)+    &
     &                                 ZooCYC(izoo)*FV1          
     
!  Phosphorus calculations

              FV1=Bio(k,i,iZooP(izoo))*ExZOO(izoo)
              Bio_new(k,i,iZooP(izoo))=Bio_new(k,i,iZooP(izoo))-    &
     &                                 FV1  

!  Excretion of DOP, FecP, PO4

              Bio_new(k,i,iDOMP(ilab))=Bio_new(k,i,iDOMP(ilab))+    &
     &                                 ZooDOC(izoo)*FV1
              Bio_new(k,i,iFecP(isfc))=Bio_new(k,i,iFecP(isfc))+    &
     &                                 ZooPEL(izoo,isfc)*FV1
              IF(Nfec==2) Bio_new(k,i,iFecP(iffc))=                 &
     &                                 Bio_new(k,i,iFecP(iffc))+    &
     &                                 ZooPEL(izoo,iffc)*FV1
              Bio_new(k,i,iPO4_)=Bio_new(k,i,iPO4_)+    &
     &                                 ZooCYC(izoo)*FV1  

     
            END DO
          END DO
	END DO
	                                       

!  Grazing.
 
        DO izoo=1,Nzoo
          DO i=1,nea
            if(idry_e(i)==1) cycle
	    DO k=kbe(i)+1, nvrt
	    
!  Carbon calculations
              
              FV1=Bio(k,i,iZooC(izoo))*GZ(izoo)
              Bio_new(k,i,iZooC(izoo))=Bio_new(k,i,iZooC(izoo))-    &
     &                                 FV1 
              Bio_new(k,i,iFecC(isfc))=Bio_new(k,i,iFecC(isfc))+    &
     &                                 ZooPEL(izoo,isfc)*FV1
              IF(Nfec==2) Bio_new(k,i,iFecC(iffc))=                 &
     &                                 Bio_new(k,i,iFecC(iffc))+    &
     &                                 ZooPEL(izoo,iffc)*FV1  
     
!  DIC

              Bio_new(k,i,iDIC_)=Bio_new(k,i,iDIC_)+    &
     &                           EfcPrd(izoo)*FV1         


!  Nitrogen calculations
              
	      FV1=Bio(k,i,iZooN(izoo))*GZ(izoo)
              Bio_new(k,i,iZooN(izoo))=Bio_new(k,i,iZooN(izoo))-    &
     &                                 FV1
      
              Bio_new(k,i,iFecN(isfc))=Bio_new(k,i,iFecN(isfc))+    &
     &                                 ZooPEL(izoo,isfc)*FV1
              IF(Nfec==2) Bio_new(k,i,iFecN(iffc))=                 &
     &                                 Bio_new(k,i,iFecN(iffc))+    &
     &                                 ZooPEL(izoo,iffc)*FV1
     
!  Phosphorus calculations

              FV1=Bio(k,i,iZooP(izoo))*GZ(izoo)
              Bio_new(k,i,iZooP(izoo))=Bio_new(k,i,iZooP(izoo))-    &
     &                                 FV1  
     
              Bio_new(k,i,iFecP(isfc))=Bio_new(k,i,iFecP(isfc))+    &
     &                                 ZooPEL(izoo,isfc)*FV1
              IF(Nfec==2) Bio_new(k,i,iFecP(iffc))=                 &
     &                                 Bio_new(k,i,iFecP(iffc))+    &
     &                                 ZooPEL(izoo,iffc)*FV1     
    
            END DO
	  END DO
	END DO    
 
!-----------------------------------------------------------------------

!
!-----------------------------------------------------------------------
!  Fecal pellet remineralization.
!-----------------------------------------------------------------------
!
          DO ifec=1,Nfec
            DO i=1,nea
              if(idry_e(i)==1) cycle
              DO k=kbe(i)+1,nvrt

!
!  Carbon calculations.  All carbon goes to CO2.
!
                FV3=Regen_C(k,i,ifec)*Bio(k,i,iFecC(ifec))
                Bio_new(k,i,iFecC(ifec))=Bio_new(k,i,iFecC(ifec))-      &
     &                                   FV3
                Bio_new(k,i,iDIC_)=Bio_new(k,i,iDIC_)+                  &
     &                             FV3

!
!  Nitrogen calculations.  Nitrogen goes to NH4.
!
                FV2=Regen_N(k,i,ifec)*Bio(k,i,iFecN(ifec))
                Bio_new(k,i,iFecN(ifec))=Bio_new(k,i,iFecN(ifec))-      &
     &                                   FV2
                Bio_new(k,i,iNH4_)=Bio_new(k,i,iNH4_)+                  &
     &                             FV2
!
!  Silica calculations.
!
                FV2=Regen_S(k,i,ifec)*Bio(k,i,iFecS(ifec))
                Bio_new(k,i,iFecS(ifec))=Bio_new(k,i,iFecS(ifec))-      &
     &                                   FV2
                Bio_new(k,i,iSiO_)=Bio_new(k,i,iSiO_)+                  &
     &                             FV2
!
!  Phosphorous calculations.
!
                FV2=Regen_P(k,i,ifec)*Bio(k,i,iFecP(ifec))
                Bio_new(k,i,iFecP(ifec))=Bio_new(k,i,iFecP(ifec))-      &
     &                                   FV2
                Bio_new(k,i,iPO4_)=Bio_new(k,i,iPO4_)+                  &
     &                             FV2
!
! MFR - Only if Fe is being calculated...
!
                IF(IRON==1)THEN

!  Iron calculations.
!
                  FV2=Regen_F(k,i,ifec)*Bio(k,i,iFecF(ifec))
                  Bio_new(k,i,iFecF(ifec))=Bio_new(k,i,iFecF(ifec))-      &
       &                                   FV2
                  Bio_new(k,i,iFeO_)=Bio_new(k,i,iFeO_)+                  &
       &                             FV2

                 END IF  !IRON
              END DO
            END DO
          END DO

! Marta Rodrigues (! To check later...)
! ----------------------------------------------------------------------
! Fecal matter sinking.
! ----------------------------------------------------------------------

          DO ifec=1,Nfec
            DO i=1,nea
              if(idry_e(i)==1) cycle
              DO k=kbe(i)+1,nvrt-1

              Bio_new(k,i,iFecC(ifec))=Bio_new(k,i,iFecC(ifec))+&
                                    -WF(ifec)*((Bio(k+1,i,iFecC(ifec))- &
                                    Bio(k-1,i,iFecC(ifec)))/(ze(k+1,i)-   &
                                    ze(k-1,i)))

              Bio_new(k,i,iFecN(ifec))=Bio_new(k,i,iFecN(ifec))+&
                                    -WF(ifec)*((Bio(k+1,i,iFecN(ifec))- &
                                    Bio(k-1,i,iFecN(ifec)))/(ze(k+1,i)-   &
                                    ze(k-1,i)))

              Bio_new(k,i,iFecP(ifec))=Bio_new(k,i,iFecP(ifec))+&
                                    -WF(ifec)*((Bio(k+1,i,iFecP(ifec))- &
                                    Bio(k-1,i,iFecP(ifec)))/(ze(k+1,i)-   &
                                    ze(k-1,i)))

              IF(IRON==1)THEN
               Bio_new(k,i,iFecF(ifec))= Bio_new(k,i,iFecF(ifec))+&
                                     -WF(ifec)*((Bio(k+1,i,iFecF(ifec))- &
                                     Bio(k-1,i,iFecF(ifec)))/(ze(k+1,i)-   &
                                     ze(k-1,i)))
              END IF

              Bio_new(k,i,iFecS(ifec))=Bio_new(k,i,iFecS(ifec))+&
                                      -WF(ifec)*((Bio(k+1,i,iFecS(ifec))- &
                                      Bio(k-1,i,iFecS(ifec)))/(ze(k+1,i)-   &
                                      ze(k-1,i)))

           END DO
          END DO 
        END DO ! Nfec

! ----------------------------------------------------------------------

!
!-----------------------------------------------------------------------
!  CDMC photolysis calculations.
!-----------------------------------------------------------------------
!
          IF (RtUVR_flag) THEN
            DO i=1,nea
              if(idry_e(i)==1) cycle
!
!  If Ed_nz(i,N(ng)) > zero, then there is sunlight. Standardizing rate
!  to 1500 umol quanta m-2 s-1.
!
              IF (Ed_nz(i,nvrt).ge.0.01) THEN
!
                FV1=RtUVR_DIC*Ed_nz(i,nvrt)/1500.0_r8
                FV2=RtUVR_DOC*Ed_nz(i,nvrt)/1500.0_r8

!
!  FV4 equals the CDMC1 absorption at 410 nm. 0.012 converts to g m-3.
!  FV5 equals the CDMC2 absorption at 410 nm.
!  Weighted average attenuation of UVB of water at 300 nm = 0.2 m-1.
!
                FV4=Bio(i,nvrt,iCDMC(ilab))*0.012_r8*aDOC410(ilab)
                FV5=Bio(i,nvrt,iCDMC(irct))*0.012_r8*aDOC410(irct)
                photo_decay=0.5_r8*Hz(i,nvrt)*                       &
     &                      (0.2_r8+(FV4+FV5)*aDOC300(ilab))
                FV3=EXP(-photo_decay)
                photo_decay=2.0_r8*photo_decay
!
!  Do not photolyze below the euphotic zone.
!
                DO k=nvrt,Keuphotic(i),-1 !número de verticais
                  IF (FV3.gt.0.01_r8) THEN
                    FV6=FV5+FV4
                    IF (FV6.gt.0.0_r8) THEN
                      FV7=FV4/FV6
                      photo_DIC=FV3*FV1*FV6
                      photo_DOC=FV3*FV2*FV6
                      total_photo=photo_DIC+photo_DOC
!
!  NOTE: not testing for excess photolysis (CDOC going negative).
!
                      FV4=(1.0_r8-FV7)*total_photo
                      Bio_new(k,i,iCDMC(irct))=Bio_new(k,i,iCDMC(irct))-&
     &                                         FV4
                      Bio_new(k,i,iDOMC(ilab))=Bio_new(k,i,iDOMC(ilab))+&
     &                                         photo_DOC
                      Bio_new(k,i,iCDMC(ilab))=Bio_new(k,i,iCDMC(ilab))-&
     &                                         FV7*total_photo
                      Bio_new(k,i,iDIC_)=Bio_new(k,i,iDIC_)+            &
     &                                         photo_DIC
                    END IF
!
!  FV4 equals the CDMC1 absorption at 410 nm. 0.012 converts to g m-3.
!  FV5 equals the CDMC2 absorption at 410 nm.
!  Weighted average attenuation of UVB of water at 300 nm = 0.2 m-1.
!
                    FV4=Bio(k,i,iCDMC(ilab))*0.012_r8*aDOC410(ilab)
                    FV5=Bio(k,i,iCDMC(irct))*0.012_r8*aDOC410(irct)
                    FV7=photo_decay+                                    &
     &                 0.5_r8*Hz(k,i)*(0.2_r8+(FV4+FV5)*aDOC300(ilab))
!
!  If k is greater than the bottom of the euphotic zone (and by
!  by extension the bottom boundary) or the decay constant is
!  greater than 4.61 (or < 1% photolysis zone) then exit do loop.
!
                    FV3=EXP(-FV7)
!
!  Store value for passage through entire Hz(i,j,k).
!
                    photo_decay=photo_decay+2.0_r8*FV7
                  END IF
                END DO
              END IF
            END DO
          END IF
!
!-----------------------------------------------------------------------
!  Create optimal pigment ratios.
!-----------------------------------------------------------------------
!
          DO i=1,nea
            if(idry_e(i)==1) cycle
            IF (Keuphotic(i).le.nvrt) THEN
              DO iphy=1,Nphy
!
!  Carbon to chlorophyll a ratio
!  This statement says that nutrient limitation of C2CHL ratio overides
!  light adaptation. Minimum of two functions may be more ecologically
!  accurate?
!
                DO k=nvrt,Keuphotic(i),-1
                  IF (b_C2Cn(iphy).lt.0.0_r8+SMALL) THEN
                    C2CHL_w(k,iphy)=MIN((b_C2Cl(iphy)+               &
     &                                   mxC2Cl(iphy)*E0_nz(k,i)),   &
     &                                  C2CHL_max(iphy))
                  ELSE IF (C2nALG(k,i,iphy).gt.                         &
     &                     minC2nALG(iphy)+SMALL) THEN
                    C2CHL_w(k,iphy)=b_C2Cn(iphy)+                    &
     &                              mxC2Cn(iphy)*                    &
     &                              (C2nALG(k,i,iphy)-                  &
     &                               minC2nALG(iphy))
                  ELSE
                    C2CHL_w(k,iphy)=MIN((b_C2Cl(iphy)+               &
     &                                   mxC2Cl(iphy)*E0_nz(k,i)),   &
     &                                  C2CHL_max(iphy))
                  END IF
                END DO
!
!  Chlorophyll a concentation per species. form g CHL a / g C
!
                DO k=nvrt,Keuphotic(i),-1
                  Pigs_w(k,iphy,ichl)=1.0_r8/C2CHL_w(k,iphy)
                END DO
!
!  Chlorophyll b concentration per species. form g CHL b / g C
!
                IF (iPigs(iphy,2).gt.0) THEN
                  DO k=nvrt,Keuphotic(i),-1
                    Pigs_w(k,iphy,2)=b_ChlB(iphy)+                   &
     &                               mxChlB(iphy)*                   &
     &                               (C2CHL_w(k,iphy)-                  &
     &                                b_C2Cl(iphy))
                    Pigs_w(k,iphy,2)=Pigs_w(k,iphy,2)*                  &
     &                               Pigs_w(k,iphy,ichl)
                  END DO
                END IF
!
!  Chlorophyll c concentration per species. form g CHL c / g C
!
                IF (iPigs(iphy,3).gt.0) THEN
                  DO k=nvrt,Keuphotic(i),-1
                    Pigs_w(k,iphy,3)=b_ChlC(iphy)+                   &
     &                               mxChlC(iphy)*                   &
     &                               (C2CHL_w(k,iphy)-                  &
     &                                 b_C2Cl(iphy))
                    Pigs_w(k,iphy,3)=Pigs_w(k,iphy,3)*                  &
     &                               Pigs_w(k,iphy,ichl)
                  END DO
                END IF
!
!  Photosynthetic caroteniods per species. form g PSC / g C
!
                IF (iPigs(iphy,4).gt.0) THEN
                  DO k=nvrt,Keuphotic(i),-1
                    Pigs_w(k,iphy,4)=b_PSC(iphy)+                    &
     &                               mxPSC(iphy)*                    &
     &                               (C2CHL_w(k,iphy)-                  &
     &                                b_C2Cl(iphy))
                    Pigs_w(k,iphy,4)=Pigs_w(k,iphy,4)*                  &
     &                               Pigs_w(k,iphy,ichl)
                  END DO
                END IF
!
!  Photoprotective caroteniods per species. form g PPC / g C
!
                IF (iPigs(iphy,5).gt.0) THEN
                  DO k=nvrt,Keuphotic(i),-1
                    Pigs_w(k,iphy,5)=b_PPC(iphy)+                    &
     &                               mxPPC(iphy)*                    &
     &                               (C2CHL_w(k,iphy)-                  &
     &                                b_C2Cl(iphy))
                    Pigs_w(k,iphy,5)=Pigs_w(k,iphy,5)*                  &
     &                               Pigs_w(k,iphy,ichl)
                  END DO
                END IF
!
!  Low Urobilin Phycoeurythin concentration per species. g LPUB / g C
!
                IF (iPigs(iphy,6).gt.0) THEN
                  DO k=nvrt,Keuphotic(i),-1
                    Pigs_w(k,iphy,6)=b_LPUb(iphy)+                   &
     &                               mxLPUb(iphy)*                   &
     &                               (C2CHL_w(k,iphy)-                  &
     &                                b_C2Cl(iphy))
                    Pigs_w(k,iphy,6)=Pigs_w(k,iphy,6)*                  &
     &                               Pigs_w(k,iphy,ichl)
                  END DO
                END IF
!
!  High Urobilin Phycoeurythin concentration per species (g HPUB / g C).
!
                IF (iPigs(iphy,7).gt.0) THEN
                  DO k=nvrt,Keuphotic(i),-1
                    Pigs_w(k,iphy,7)=b_HPUb(iphy)+                   &
     &                               mxHPUb(iphy)*                   &
     &                               (C2CHL_w(k,iphy)-                  &
     &                                b_C2Cl(iphy))
                    Pigs_w(k,iphy,7)=Pigs_w(k,iphy,7)*                  &
     &                               Pigs_w(k,iphy,ichl)
                  END DO
                END IF
              END DO
!
!  Calculate pigment ratio changes.
!  NOTE: 12 factor to convert to ugrams (mg m-3)
!
              DO ipig=1,Npig
                DO iphy=1,Nphy
                  IF (iPigs(iphy,ipig).gt.0) THEN
                    itrc=iPigs(iphy,ipig)
                    DO k=nvrt,Keuphotic(i),-1
                      IF ((Bio(k,i,iPhyC(iphy)).gt.0.0_r8).and.         &
     &                    (Bio(k,i,itrc).gt.0.0_r8)) THEN
                        FV1=Bio(k,i,iPhyC(iphy))*12.0_r8
                        FV2=GtALG_r(k,i,iphy)
                        FV3=FV1/                                        &
     &                      (FV2/Pigs_w(k,iphy,ipig)+                   &
     &                       FV1*(1.0_r8-FV2)/                          &
     &                       Bio(k,i,itrc))
                        Bio_new(k,i,itrc)=Bio_new(k,i,itrc)+            &
     &                                    (FV3-Bio(k,i,itrc))
                      END IF
                    END DO
                  END IF
                END DO
              END DO
            END IF
          END DO
!
!-----------------------------------------------------------------------
!  Update the tendency arrays
!-----------------------------------------------------------------------
!
          DO ibio=1,NBT
            itrc=idbio(ibio)
            DO i=1,nea
              if(idry_e(i)==1) cycle
              DO k=kbe(i)+1,nvrt
!MFR             Bio(k,i,itrc)=Bio(k,i,itrc)+dtbio*Bio_new(k,i,itrc)
                Bio_new(k,i,itrc)=Bio_new(k,i,itrc)+dtbio*Bio_new(k,i,itrc)
              END DO
            END DO
          END DO
        END DO ITER_LOOP
!
!-----------------------------------------------------------------------
!  Update global tracer variables.
!  MFR - Calculates body source for SELFE
!-----------------------------------------------------------------------
!
        DO ibio=1,NBT
          itrc=idbio(ibio)
          DO i=1,nea
            if(idry_e(i)==1) cycle
            DO k=kbe(i)+1,nvrt

! MFR ... To check tracer concentration
               Bio(k,i,itrc)=Bio_old(k,i,itrc)+Bio_new(k,i,itrc)
               IF(Bio(k,i,itrc)<MinVal)THEN
                 bdy_frc(itrc,k,i)=0.d0 !MFR - Check later...
               ELSE 
                 bdy_frc(itrc,k,i)=Bio_new(k,i,itrc)/dt
               END IF

! MFR               t(k,i,itrc)=MAX(MinVal,                           &
! MFR     &                                t(k,i,itrc)+               &
! MFR     &                                Hz(k,i)*                        &
! MFR     &                                (Bio(k,i,itrc)-Bio_old(k,i,itrc)))

            END DO
          END DO
        END DO

! Marta Rodrigues
! ---------------------------------------------------------------------
! Calculates bottom flux for SELFE
! ---------------------------------------------------------------------
!       flx_bt(mntr,nea)
        flx_bt = 0.d0
!
! ---------------------------------------------------------------------
! Calculates surface flux for SELFE
! ---------------------------------------------------------------------
!       flx_sf(mntr,nea)
        flx_sf = 0.d0
!
! ---------------------------------------------------------------------

      RETURN
      END SUBROUTINE
